  # proc _dbg {msg} {puts stderr $msg}
  proc _dbg {msg} {}
  proc _servIt {checkIt channel} {
    global cmdPrefix

    if {[eof $channel]} {
      _dbg "'$channel is closing'..."
      close $channel
      return
    }

    if {[gets $channel cmd] < 0} return

    if {[info exists cmdPrefix]} {
      if {[string compare $cmd "TCL RPC END"]} {
        _dbg "Found valid command line $cmd..."
        set sep [expr {[string length $cmdPrefix] ? "\n" : ""}]
        append cmdPrefix $sep $cmd
        return
      } else {
        _dbg "Found RPC End..."
        set cmd $cmdPrefix
      }
      unset cmdPrefix
    } elseif {![string compare $cmd "TCL RPC START"]} {
      _dbg "Found RPC Start..."
      set cmdPrefix ""
      return
    } else {
      _dbg "Found '$cmd' outside a TCL RPC..."
      puts $channel "TCL ERROR not a rpc"
      return
    }

    if {![$checkIt $cmd]} {
      _dbg "Invalid command '$cmd'..."
      puts $channel "TCL ERROR invalid arg"
      flush $channel
      return
    }

    puts $channel "TCL RESULT START"
    _dbg "Calling command '$cmd'..."
    if {[catch {uplevel #0 $cmd} msg]} {
      _dbg "'$cmd' returned error $msg..."
      puts $channel "ERROR: $msg"
    } else {
      _dbg "'$cmd' returned $msg..."
      puts $channel $msg
    }
    puts $channel "TCL RESULT END"
    flush $channel
  }
  proc _connectIt {checkIt channel address port} {
    fconfigure $channel -blocking 0
    fileevent $channel readable "_servIt $checkIt $channel"
  }
  proc _noCheck {args} {
    return 1
  }
  proc dp_MakeRPCServer {{port 0} {loginProc none} {checkIt none} {retFile 0}} {
    if {![string compare $checkIt none]} {set checkIt _noCheck}
    set serverFile [socket -server "_connectIt $checkIt" $port]
    set serverPort [lindex [fconfigure $serverFile -sockname] 2]
    _dbg "server(port=$port)==>(file='$serverFile',port='$serverPort')"

    if {$retFile} {
      return [list $serverFile $serverPort]
    } else {
      return $serverPort
    }
  }
  proc dp_MakeRPCClient {host port} {
    if {[catch {socket $host $port} channel]} {
      return -code error $channel
    }
    return $channel
  }
  proc dp_RPC {channel args} {
    puts $channel "TCL RPC START"
    eval puts $channel [list $args]
    puts $channel "TCL RPC END"
    flush $channel

    set status beforeStart
    set result ""
    while {[string compare $status afterEnd]} {
      set row [gets $channel]
      if {[string match "TCL ERROR*" $row]} {
        return -code error $row
      }
      switch $status {
        beforeStart {
          if {![string compare $row "TCL RESULT START"]} {
            set status result
          }
        }
        result {
          if {[string compare $row "TCL RESULT END"]} {
            set sep [expr {[string length $result] ? "\n" : ""}]
            append result $sep $row
          } else {
            set status afterEnd
          }
        }
      }
    }
    _dbg "result is '$result'..."
    return $result
  }
  proc dp_CloseRPC {channel} {
    close $channel
  }
