  
  # ------------------------------------------------------------------------
  # Copyright (c) 1995 Christian Krone, Varziner Str. 12, D-12161 Berlin
  # All rights reserved.
  # ------------------------------------------------------------------------
  # Permission is hereby granted, without written agreement and without
  # license or royalty fees, to use, copy, modify, and distribute this
  # software and its documentation for any purpose, provided that the
  # above copyright notice and the following two paragraphs appear in
  # all copies of this software.
  # ------------------------------------------------------------------------
  # IN NO EVENT SHALL CHRISTIAN KRONE BE LIABLE TO ANY PARTY FOR DIRECT,
  # INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
  # OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF CHRISTIAN
  # KRONE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  # ------------------------------------------------------------------------
  # CHRISTIAN KRONE SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT
  # NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  # FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN
  # "AS IS" BASIS, AND CHRISTIAN KRONE HAS NO OBLIGATION TO PROVIDE
  # MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
  # ------------------------------------------------------------------------
    proc kopiereRegeln {dest src} {
        foreach regel {kartenSatz stapelAnz patName stapelReihe stockBasis
                       stockReihe strafReihe streitArt leerStapelNurHoch
                       talonAbraeumbar durchgaenge teilStapelReihe exist 
                       stapelGrund stapelVerdeckt} {
          upvar #0 $regel var

          foreach name [array names var] {
            if {![string compare $src $name]} {
              set var($dest) $var($name)
            } elseif {[regexp ^${src},(.*)$ $name egal subIndex]} {
              set var($dest,$subIndex) $var($name)
            }
          }
        }
      }
    proc liesSpiel {} {
      global patArt patName kartenSatz streitArt exist stapelAnz \
             stapelGrund stapelVerdeckt stapelReihe stockBasis   \
             stockReihe teilStapelReihe strafReihe               \
             leerStapelNurHoch talonAbraeumbar durchgaenge
      global patiencePath neuPat
      foreach spielDir [list $patiencePath/spiel \
                             [split [option get .patience sprPath SprPath] :]] {
        foreach spiel [glob -nocomplain [fullPath $spielDir *.{spr,SPR}]] {
          source $spiel
        }
      }
      if {![info exists patArt]} {
        set patArt standard
        set patName($patArt,deutsch)    "Standard"
        set patName($patArt,english)    "Standard"
        set kartenSatz($patArt)         2
        set streitArt($patArt)          0
        set exist($patArt,talon)        0
        set exist($patArt,strafe)       0
        set exist($patArt,arbeit)       0
        set stapelAnz($patArt)          8
        set stapelGrund($patArt,0)      3
        set stapelGrund($patArt,1)      3
        set stapelGrund($patArt,2)      3
        set stapelGrund($patArt,3)      3
        set stapelGrund($patArt,4)      3
        set stapelGrund($patArt,5)      3
        set stapelGrund($patArt,6)      3
        set stapelGrund($patArt,7)      3
        set stapelVerdeckt($patArt,0)   2
        set stapelVerdeckt($patArt,1)   2
        set stapelVerdeckt($patArt,2)   2
        set stapelVerdeckt($patArt,3)   2
        set stapelVerdeckt($patArt,4)   2
        set stapelVerdeckt($patArt,5)   2
        set stapelVerdeckt($patArt,6)   2
        set stapelVerdeckt($patArt,7)   2
        set stapelReihe($patArt)        abst,unecht
        set teilStapelReihe($patArt)    abst,unecht
        set stockBasis($patArt)         a
        set stockReihe($patArt)         aufst,echt
        set strafReihe($patArt)         aufAb,echt
        set leerStapelNurHoch($patArt)  0
        set talonAbraeumbar($patArt)    1
        set durchgaenge($patArt)        0
      }
      set vorgabe [winfo name .]
      set gefunden 0
      foreach bekannteRegel [array names stapelAnz] {
        if {[string match $bekannteRegel* $vorgabe]} {
          set patArt $bekannteRegel
          set gefunden 1
        }
      }
      if {!$gefunden} {
        set patArt standard
      }
      set neuPat $patArt
    }
    proc schrRegeln {fileName schrPat} {
      global patArt patName kartenSatz streitArt exist stapelAnz \
             stapelGrund stapelVerdeckt stapelReihe stockBasis   \
             stockReihe teilStapelReihe strafReihe               \
             leerStapelNurHoch talonAbraeumbar durchgaenge

      lappend zust "set patArt [string trimleft $schrPat :]"
      foreach regel {kartenSatz stapelAnz patName stapelReihe stockBasis
                     stockReihe strafReihe streitArt leerStapelNurHoch
                     talonAbraeumbar durchgaenge teilStapelReihe exist
                     stapelGrund stapelVerdeckt} {
        upvar #0 $regel var

        foreach name [array names var] {
          if {![string compare $schrPat $name]} {
            lappend zust "set ${regel}(\$patArt) \"$var($name)\""
          } elseif {[regexp ^${schrPat},(.*)$ $name egal subIndex]} {
            lappend zust "set ${regel}(\$patArt,$subIndex) \"$var($name)\""
          }
        }
      }

      set aus [open $fileName "w"]
      global patienceVersion
      puts $aus "# ------------------------------------------------------------"
      puts $aus "#    Patience Version $patienceVersion"
      puts $aus "#    Spielregeln erzeugt durch 'Regeln editieren/Speichern'"
      puts $aus "#    Per Hand aendern ist unsportlich UND wird geahndet."
      puts $aus "# ------------------------------------------------------------"
      foreach zeile $zust {
        puts $aus $zeile
      }
      close $aus
    }
    proc initZustand {} {
        global stapel fuelle verdeckt teilStapelUnterst streitArt patArt \
               inversZug vorschlaege vorschlagZaehler mogelZaehler zugNr runden

        foreach var {stapel fuelle verdeckt vorschlaege
                     teilStapelUnterst aktiverSpieler} {
          catch "unset $var"
        }
        set fuelle(hand2) " "
        trace variable fuelle(hand)  w chkKartenAnz
        trace variable fuelle(hand2) w chkKartenAnz

        set inversZug        {}
        set vorschlagZaehler 0
        set mogelZaehler     0
        set zugNr            0
        set runden           0
      }
    proc schrZustand {fileName invers} {
      global patienceVersion exist streitArt patArt stapelAnz kartenSatz
      global zugNr mogelZaehler inversZug vorschlagZaehler hand runden \
             stapel fuelle verdeckt teilStapelUnterst aktuellErste aktiverSpieler

      upvar $fileName zust
      set zust ""

      global tk_version patienceVersion
      lappend zust "# -------------------------------------------------------------"
      lappend zust "#    Patience Version $patienceVersion"
      lappend zust "#    Programmzustand erzeugt durch die Funktion 'Speichern'"
      lappend zust "#    Per Hand aendern ist unsportlich UND wird geahndet."
      lappend zust "# -------------------------------------------------------------"
      lappend zust ""
      lappend zust "global tk_version patienceVersion"
      lappend zust "if {\$tk_version      < $tk_version ||"
      lappend zust "    \$patienceVersion < $patienceVersion} return"

      set schrPat $patArt
      lappend zust "set patArt [string trimleft $schrPat :]"
      foreach regel {kartenSatz stapelAnz patName stapelReihe stockBasis
                     stockReihe strafReihe streitArt leerStapelNurHoch
                     talonAbraeumbar durchgaenge teilStapelReihe exist
                     stapelGrund stapelVerdeckt} {
        upvar #0 $regel var

        foreach name [array names var] {
          if {![string compare $schrPat $name]} {
            lappend zust "set ${regel}(\$patArt) \"$var($name)\""
          } elseif {[regexp ^${schrPat},(.*)$ $name egal subIndex]} {
            lappend zust "set ${regel}(\$patArt,$subIndex) \"$var($name)\""
          }
        }
      }
      lappend zust ""

      lappend zust "global zugNr;            set zugNr            $zugNr"
      lappend zust "global runden;           set runden           $runden"
      lappend zust "global mogelZaehler;     set mogelZaehler     $mogelZaehler"
      lappend zust "global vorschlagZaehler; set vorschlagZaehler $vorschlagZaehler"
      lappend zust "global aktuellErste;     set aktuellErste     $aktuellErste"
      if {[info exists aktiverSpieler]} {
        lappend zust "global aktiverSpieler; set aktiverSpieler   $aktiverSpieler"
      }
      lappend zust "global stapel fuelle verdeckt teilStapelUnterst"
      foreach stap {stapel fuelle verdeckt teilStapelUnterst} {
        if {$invers} {
          set tmpZust ""
          pArray tmpZust $stap
          regsub -all (hand|talon|strafe)2  $tmpZust {\13} tmpZust
          regsub -all (hand|talon|strafe)   $tmpZust {\12} tmpZust
          regsub -all (hand|talon|strafe)23 $tmpZust {\1}  tmpZust
          append zust " $tmpZust"
        } else {
          pArray zust $stap
        }
      }
      if {!$streitArt($patArt)} {
        lappend zust "global inversZug;    set inversZug [list $inversZug]"
      }
    }
    proc liesZustand {zustand art} {
      global patArt patName kartenSatz streitArt exist stapelAnz \
             stapelGrund stapelVerdeckt stapelReihe stockBasis   \
             stockReihe teilStapelReihe strafReihe               \
             leerStapelNurHoch talonAbraeumbar durchgaenge
      global neuPat text

      set altArt $patArt

      initZustand

      switch $art {
        datei {source $zustand}
        var   {eval   [join $zustand "\n"]}
      }

      set patArt $patArt
      set neuPat $patArt
      
      setBitPath <>
      if {$patArt != $altArt} {
        aktiviereMenuepunkteFuerStreit
        aktiviereBindings "#"
        aktiviereBindings
        highScore 0
      }
      
      if {[info exists aktiverSpieler]
          && ![string compare $aktiverSpieler $text(computer)]} {
        demo
      }
    }
    proc pArray {aus a} {
    # global tk_version
      upvar #0 $a array
      upvar $aus zust

    # if {$tk_version >= 4.0} {
    #   lappend zust "array set $a {[array get array]}"
    #   return
    # }
      foreach name [array names array] {
        lappend zust [format "  set %s(%s) %s" $a $name [list $array($name)]]
      }
    }
    proc liesTexte {} {
        global text flaeche patiencePath sprache sprachDefault

        set sprache [option get .patience sprache Sprache]
        if {[catch "source $patiencePath/text/$sprache"]} {
          englishDefault
          set text(patience) "Patience"
          set text(bestWertEingeben) "Please enter the dates for the high score list:"
          set text(spielername)      "Name of player"
          set text(zeit)             "Date/time"
          set text(spiel)           " Games "
          set text(streitMit)       "Fight with "
          set text(mensch)          "Human"
          set text(computer)        "Computer"
          set text(streitBereit)    "Ready to compete"
          set text(kontaktAufnahme) "Establish contact"
          set text(kontaktPruefen)  "Check contact"
          set text(kontaktAbgleich) "Synchronize contact"
          set text(demo)            "Demo"
          set text(warte)           "Wait "
          set text(sekunden)        " sec"
          set text(bildErneuern)    "Refresh screen"
          set text(beenden)         "Quit"
          set text(acc,neuesSpiel)  "n"
          set text(acc,demo)        "d"
          set text(acc,beenden)     "q"
          set text(datei)           " File "
          set text(spielzustand)    "State of game"
          set text(laden)           "Load"
          set text(speichern)       "Store"
          set text(highScore)       "Hall of Fame"
          set text(regelnEditieren) "Edit rules"
          set text(sprache)         "Language"
          set text(acc,laden)       "l"
          set text(acc,speichern)   "s"
          set text(acc,highScore)   "f"
          set text(sprache) "Language"
          set text(unterstuetzung)  " Specials "
          set text(vorschlag)       "Hints"
          set text(ablegen)         "Move to stacks"
          set text(zugZurueck)      "Undo one move"
          set text(zumAnfang)       "Start over"
          set text(merken)          "Mark"
          set text(zurMarke)        "Back to mark"
          set text(veraendern)      "Cheating"
          set text(acc,vorschlag)   "h"
          set text(acc,ablegen)     "m"
          set text(acc,zugZurueck)  "u"
          set text(karten)          " Cards "
          set text(geben)           "New cards"
          set text(noch)            "Still "
          set text(kartenUebrig)    " cards"
          set text(kartenfonts)     "Cardfonts"
          set text(mitComics)       "With comics"
          set text(verteilen)       "Shuffle"
          set text(kartenbewegen)   "Movements"
          set text(schatten:)       "Shadow height: "
          set text(acc,geben)       "c"
          set text(hilfe)              " Help "
          set text(erstellt)           "About Patience"
          set text(title,einleitung)   "Introduction"
          set text(title,regeln)       "Rules"
          set text(title,kommandos)    "Commands"
          set text(title,streit)       "Battlesolitaire"
          set text(title,regeledit)    "The Ruleeditor"
          set text(title,installation) "Installation"
          set text(title,wuerdigungen) "Credits"
          set text(file,einleitung)    "Introduc"
          set text(file,regeln)        "Rules"
          set text(file,kommandos)     "Commands"
          set text(file,streit)        "Battleso"
          set text(file,regeledit)     "TheRulee"
          set text(file,installation)  "Installa"
          set text(file,wuerdigungen)  "Credits"
          set text(netscape)           "Use Netscape"

          set text(demoHilfe)    "Any key quits the demo"
          set text(streitHilfe)  "The key q or n quits the game"
          set text(zurueckHilfe) "Any key quits the undo"
          set text(speicherAuswahl) "Fileselection for store"
          set text(ladeAuswahl)     "Fileselection for load"
          set text(fuer)          " for "
          set text(bst:nr)        "Rank"
          set text(bst:spieler)   "Player"
          set text(bst:zuege)     "Moves"
          set text(bst:vorschlag) "Hints"
          set text(bst:karten)    "Rem. cards"
          set text(bst:du)        "Opponent"
          set text(bst:zeit)      "Date/Time"
          set text(bildLesen) "Reading image"
          set text(nixHtml)   "No formatted output, since htmllib isn't available..."
          set text(einlesen)  "Parsing file"
          set text(anzeigen)  "Rendering file"
          set text(fertig)    "done"
          set text(schatten)        "Shadow"
          set text(schattenGroesse) "Height of moving cards (in screen points)"
          set text(hinweis:schachUhr) "Press button below,
          if no more moves are possible
          and the partner should take over"
          set text(hinweis:ablegen) "Hint: The command Move to stacks
          in menu Specials puts cards automatically
          to the stacks, if possible"
          set text(demonstration)    "Demonstration"
          set text(zeitverzoegerung) "Time delay (in 1/10 seconds)"
          set text(sendeFehler)   "Error while sending"
          set text(keineReaktion) "Other Patience don't react any more."
          set text(neuSenden)     "Send again"
          set text(neuAnfragen)   "Call anew"
          set text(gegenComputer) "Continue against computer"
          set text(kontakt)            "Looking for a partner"
          set text(willDochNicht)      "Partner doesn't want to play anymore"
          set text(nichtEinverstanden) "Partner doesn't agree"
          set text(anfragen)           "Call"
          set text(abgleich)           "Synchronize"
          set text(gegnerFragen)       "Ask for confirmation of partner..."
          set text(zustandUebernehmen) "Okay to accept the state of game of partner?"
          set text(aufgabe)          "Surrender game"
          set text(kontaktAbbrechen) "Really stop the contact with partner?"
          set text(kontaktNeustart)  "Really surrender the current game?"
          set text(neuerAnfang) "Partner surrenders. Ready for a new game?"
          set text(gegnerAngeben)   "Please give name, host and port of partner."
          set text(gegnerSuchen)    "Looking for desired partner..."
          set text(gegnerAnrufen)   "Calling desired partner..."
          set text(gegnerUnwillig)  "Desired partner doesn't like to compete right now."
          set text(gegnerNichtDa)   "Desired partner doesn't play Patience right now."
          set text(gegnerUnbekannt) "Desired partner unknown at given host."
          set text(gegenSelbst)     "You can't compete with yourself."
          set text(keinDaemonDa)    "At given host nobody likes to compete."
          set text(deinPeerFehlt)   "Partner doesn't react on call."
          set text(meinPortFehlt)   "No contact possible right now (my fault)."
          set text(hatAufgegeben) "Your partner surrendered."
          set text(weiterOhneIhn) "Continue as single"
          set text(spielGefaellig) "Battle solitaire?"
          set text(besetzt)        "Desired partner is already playing in a battle."
          set text(erMagSpielen)   "The given user likes to start a game with you."
          set text(hatKeineLust)   "Desired partner doesn't like to play right now."
          set text(spaeter)        "Perhaps later"
          set text(gehtLos)        "Desired partner starts right now the game."
          set text(unbekPatArt)    "Desired partner doesn't know the rules of this game."
          set text(gegnerName) "Name of partner:"
          set text(gegnerHost) "Host:"
          set text(gegnerPort) "Port:"
          set text(werFaengtAn) "Who starts?"
          set text(bstEingetragen) "Result added in high score list."
          set text(reichtNicht)    "Result displayed temporary in high score list."
          set text(regelEditor) "Rule editor"
          set text(neu)         "New"
          set text(anwenden)    "Apply"
          set text(nameIntern)  "Name (intern):"
          set text(nameExtern)  "Name (extern):"
          set text(kartensatz)  "Number of decks:"
          set text(einSatz)     "1 (52 cards)"
          set text(zweiSaetze)  "2 (104 cards)"
          set text(streitArt)   "Battle game:"
          set text(nein)        "no"
          set text(ja)          "yes"
          set text(jaGemeinsam) "yes, common cards"
          set text(jaGetrennt)  "yes, seperate cards"
          set text(talonExist)  "Reserve pile:"
          set text(strafeExist) "Penalty pile: "
          set text(arbeitExist) "Work piles:     "
          set text(stapelAnz)   "No. of piles:  "
          set text(untereOffen) "Underlying cards visible:" 
          set text(stapelReihe) "Allowed on piles:"
          set text(absteigend)  "Decreasing"
          set text(aufsteigend) "Increasing"
          set text(aufAb)       "In-/decreasing"
          set text(echt)        "same"
          set text(unecht)      "black/red"
          set text(egal)        "any"
          set text(teilStapel)  "Moveable sequence:"
          set text(einzelKarte) "Just one card"
          set text(stockReihe)  "Allowed on stack:" 
          set text(komplett)    "complete"
          set text(strafReihe)  "Allowed on penalty pile:"
          set text(stockBasis)  "Ground of stack:"
          set text(bauer)       "Jack (B)"
          set text(dame)        "Queen (D)"
          set text(koenig)      "King (K)"
          set text(as)          "Ace"
          set text(ersteKarte)  "First Card"
          set text(durchgaenge) "Turnarounds:"
          set text(beliebigOft) "Unlimited"
          set text(garNicht)    "Not at all"
          set text(stapelBasis) "Ground of pile:"
          set text(jedeKarte)   "any card"
          set text(nurHoechste) "only the highest"
          set text(talonAb)     "Old cards are moveable:"
          set text(neuerSpielregeln) "New set of rules"
          set text(nameAufforderung) \
                  "Please give a new intern name of the solitaire game:"
          set text(bisherVergeben) "Already used intern names are "
          set text(nurErlaubt) \
                  ". Allowed are letters, digits and - or _"
          set text(fs,ordner) "Directory:"
          set text(fs,datei)  "File:"
          set text(fs,maske)  "Mask:"
          set text(version)          "Version"
          set text(patienceFuerX11)  "A solitaire card game for the X11 windowsystem with Tcl/Tk"
          set text(erstelltVonCK)    "produced by Christian Krone"
          set text(okay)             "Okay"
          set text(beendenTitel)     "Really quit?"
          set text(hoffeSpielWarGut) "I hope you enjoy the solitaire game."
          set text(wirklichBeenden)  "Really quit the game?"
          set text(abbruch)          "Cancel"
          set text(spielende)        "Quit"
          set text(neuesSpiel)       "New game"
          set text(wirklichNeu)      "Do you really want to start a new game?"
          set text(glueckwunsch)     "Congratulations"
          set text(gratuliere)       "Congratulations!"
          set text(undEsGab)         "And there are "
          set text(einVorschlag)     "one suggestion."
          set text(vorschlaege)      " suggestions."
          set text(patienceGeloest)  "You won this solitaire game."
          set text(dazuBenoetigt)    "For this you needed "
          set text(zuege)            " moves."
          set text(sieHabenGewonnen) "You won this battle solitaire. "
          set text(undGegnerHat)     "And the partner still holds "
          set text(restKarte1)       " card on his piles."
          set text(restKarten1)      " cards on his piles."
          set text(beileid)          "Sorry"
          set text(schade)           "Sorry!"
          set text(sieHabenVerloren) "You loose this battle solitaire. "
          set text(undSieHatten)     "And you still hold "
          set text(restKarte2)       " card on your pile."
          set text(restKarten2)      " cards on your pile."
          set text(naechstesMal)     "Perhaps next time"
          set text(spielAufgegangen) "The cards are gone!"
          set text(aberSieHaben)     "But you cheat exactly "
          set text(malGeschummelt)   " times!"
          set text(vielleichtBesser) "Perhaps the next time you play honest?"
          set text(probieren)        "Nice try"
          set text(nichtLesbar)        "Not readable"
          set text(nichtSchreibbar)    "Not writeable"
          set text(dieDatei)           "The file "
          set text(istNichtLesbar)     " isn't readable."
          set text(istNichtSchreibbar) " isn't writeable."
          set text(dannNicht)          "Don't care"
          set text(keineVerteilung) "Only against computer"
          set text(streitNurGegenComputer)  "Note: Battle solitaires are only possible \
          against the computer, since the wish interpreting this program isn't of \
          version 4.1 or above.

          See chapter 6 of the manual (or the online-help) for further information \
          about the installation of a patience, which can compete with a player at \
          remote hosts."
          set text(keinDaemon)   "No patience daemon"
          set text(daemonFehler) "The error message above was told by the patience \
          daemon, which was started a moment ago.

          So it isn't possible for players on remote hosts to get in contact with you. \
          But it should still be manageable for you to get in contact with other \
          players."
          set text(keineHilfe) "No help available"
          set text(da)         "Since "
          set text(fehlt)      " isn't installed correctly, no help is available"
          set text(keineKartenFonts) \
                  "Oops? No Cardbitmap found; so now game is over..."
        }
        setzBenutzernamen $text(computer)
      }
    proc setzBenutzernamen {gegnerName} {
      global env ich du text

      if {[info exists env(USER)]} {
        set vorgabe $env(USER)
      } elseif {[info exists env(LOGNAME)]} {
        set vorgabe $env(LOGNAME)
      } else {
        set vorgabe $text(mensch)
      }

      set ich [capitalize $vorgabe]
      set du  $gegnerName
    }
    proc pruefSpielerNameUndDatum {} {
      global ich text jetzt tcl_platform

      if {![info exists jetzt]} {set jetzt ""}
      if {[catch {clock format [clock seconds] -format "%d %b %Y %R"} jetzt]} {
        catch {exec date} jetzt
      }

      set nameDa [string compare $ich $text(mensch)]
      set zeitDa [string length  $jetzt]

      if {!$nameDa || !$zeitDa} {
        set w [toplevel .name]
        wm title $w $text(spielername)

        label $w.text -text $text(bestWertEingeben)

        frame $w.name
        label $w.name.text -text $text(spielername)
        entry $w.name.feld -textvariable ich -width 15
        if {$nameDa} {$w.name.feld configure -state disabled}
        pack  $w.name.feld $w.name.text -side right -fill both -expand 1

        frame $w.zeit
        label $w.zeit.text -text $text(zeit)
        entry $w.zeit.feld -textvariable jetzt -width 15
        if {$zeitDa} {$w.zeit.feld configure -state disabled}
        pack  $w.zeit.feld $w.zeit.text -side right -fill both -expand 1

        button $w.ok  -text $text(okay) -command "destroy $w"
        pack   $w.text $w.name $w.zeit $w.ok -padx 4 -pady 2 -fill x -expand 1
        bind   $w.name <Return> "$w.ok flash; $w.ok invoke"

        bind  $w.name.feld <Return> "$w.ok flash; $w.ok invoke"
        bind  $w.zeit.feld <Return> "$w.ok flash; $w.ok invoke"

        set oldFocus [focus]
        grab set $w
        focus [expr {$nameDa ? "$w.zeit.feld" : "$w.name.feld"}]
        tkwait window $w
        focus $oldFocus

        if {[string length $ich]} {
          set ich [capitalize $ich]
        } else {
          set ich $text(mensch)
        }
      }
    }
    proc capitalize {wort} {
      if {[regexp {^([a-z]*|[A-Z])$} $wort]} {
        set ersterGross [string toupper [string index $wort 0]]
        set restKlein   [string tolower [string range $wort 1 end]]
        return $ersterGross$restKlein
      } else {
        return $wort
      }
    }
    proc kartenFarbe {rang} {
        switch -- $rang {
          0 {set aktFarbe c}
          1 {set aktFarbe h}
          2 {set aktFarbe p}
          3 {set aktFarbe k}
        }
      }
      proc farbRang {farbe} {
        switch -glob $farbe {
          *c* {set aktFarbe 0}
          *h* {set aktFarbe 1}
          *p* {set aktFarbe 2}
          *k* {set aktFarbe 3}
        }
      }
      proc passt {alt neu stapel} {
        global stapelReihe teilStapelReihe stockReihe strafReihe \
               patArt aktuellErste

        if {![string length $neu] && [string match ?,? $stapel]} {
          set stockFarbe [string index $stapel 2]
          return [expr {[string match *,kompl $stockReihe($patArt)]
                        ? [aufsteigend $alt $aktuellErste 1]
                          && $stockFarbe == [string index $alt 1]
                        : [string range $alt 0 1] == "$aktuellErste$stockFarbe"}]
        }
        switch -glob $stapel {
          ?                {set regel $stapelReihe($patArt)}
          ?,?              {set regel $stockReihe($patArt)}
          teil             {set regel $teilStapelReihe($patArt)}
          talon* - strafe* {set regel $strafReihe($patArt)}
          default          {return 0}
        }
        switch -glob $regel {
          aufst,* {if {![aufsteigend $neu $alt 0]} {return 0}}
          abst,*  {if {![aufsteigend $alt $neu 0]} {return 0}}
          aufAb,* {if {![aufsteigend $alt $neu 0] &&
                       ![aufsteigend $neu $alt 0]} {return 0}}

          garNicht {return 0}
        }
        switch -glob $regel {
          *,unecht {if {![unecht $alt $neu]} {return 0}}
          *,echt   {if {![  echt $alt $neu]} {return 0}}
          *,kompl  {return 0}
        }
        return 1
      }
    proc mischen {} {
      global stapel fuelle kartenSatz patArt streitArt streitVorgabe

      if {[info exists streitVorgabe]} {
        global streitVorgabe stapel fuelle aktiverSpieler ich du kontakt

        set du                   $streitVorgabe(du)
        set kontakt(deinRechner) $streitVorgabe(deinRechner)
        set stapel(meinStart)    $streitVorgabe(meinStart)
        set stapel(deinStart)    $streitVorgabe(deinStart)

        foreach srcDest {{deineHand hand} {meineHand hand2}} {
          set src  [lindex $srcDest 0]
          set dest [lindex $srcDest 1]

          set fuelle($dest) [lindex $streitVorgabe($src) 0]
          set streitVorgabe($src) [lrange $streitVorgabe($src) 1 end]
          for {set aktI 0} {$aktI < $fuelle($dest)} {incr aktI} {
            set stapel($dest,$aktI) [lindex $streitVorgabe($src) $aktI]
          }
        }
        return
      }

      foreach runde {0 1} {
        set karten$runde {}
        if {$runde < $kartenSatz($patArt)} {
          foreach wert {k d b 0 9 8 7 6 5 4 3 2 a} {
            foreach farbe {c h p k} {
              lappend karten$runde $wert$farbe$runde
            }
          }
        }
      }

      if {$streitArt($patArt)} {
        set stapel(meinStart) ""
        set stapel(deinStart) ""
        while {$stapel(meinStart) == $stapel(deinStart)} {
          set stapel(meinStart) [lindex $karten0 [unifRand 52]]
          set stapel(deinStart) [lindex $karten0 [unifRand 52]]
        }
      }

      if {$streitArt($patArt) != 2} {
        set karten [concat $karten0 $karten1]
      }

      foreach hand [expr {$streitArt($patArt) ? "hand hand2" : "hand"}] {
        if {$streitArt($patArt) == 2} {
          set karten [expr {$hand=="hand2" ? $karten1 : $karten0}]
        }
        set fuelle($hand) 0
        set nochDa [llength $karten]
        while {$nochDa > 0} {
          set oben [unifRand $nochDa] 
          set stapel($hand,$fuelle($hand)) [lindex $karten $oben]
          set karten [lreplace $karten $oben $oben]
          incr fuelle($hand)
          incr nochDa -1
        }
      }
    }
    proc mkMenue {} {
        global patArt fuelle patiencePath patName streitArt zugNr flaeche ich du \
            kartenFont kartenFonts text sprache warteZeit schatten tk_version

        wm iconbitmap . @$patiencePath/bitmaps/icon
        frame .menu -relief raised -borderwidth 1
        pack  .menu -fill x
        menubutton .menu.pat -text $text(spiel) -menu .menu.pat.m -underline 1
        menu .menu.pat.m

        set streitPatienceDa 0
        foreach aktPat [interneRegelNamen] {
          if {[info exists patName($aktPat,$sprache)]} {
            if {$streitArt($aktPat)} {
              set streitPatienceDa 1
            } else {
              .menu.pat.m add radio -label $patName($aktPat,$sprache) \
                          -var neuPat -val $aktPat -command "starteSpiel 1"
            }
          }
        }
        if {$streitPatienceDa} {
          .menu.pat.m add separator
          .menu.pat.m add command -label $text(streitMit)$du -state disabled
          foreach aktPat [interneRegelNamen] {
            if {[info exists patName($aktPat,$sprache)] && $streitArt($aktPat)} {
              .menu.pat.m add radio -label $patName($aktPat,$sprache) \
                          -var neuPat -val $aktPat -command "starteSpiel 1"
            }
          }
          .menu.pat.m add command -label $text(kontaktAufnahme)... -command kontakt
          .menu.pat.m add command -label $text(kontaktPruefen)...  -command ping
          .menu.pat.m add command -label $text(kontaktAbgleich)... -command abgleich
          .menu.pat.m add check   -label $text(streitBereit) -variable streitbar \
                                  -command streitbar
        }
        .menu.pat.m add separator
        .menu.pat.m add command -lab $text(demo) -acc $text(acc,demo) \
                                -com "demo" -und 0
        set zehntel [expr {double($warteZeit)/10}]
        .menu.pat.m add command -lab "$text(warte)$zehntel$text(sekunden)..." \
                                -com "warteZeit" -und 0
        .menu.pat.m add separator
        .menu.pat.m add command -lab $text(bildErneuern) -com "bildNeu" -acc ^L
        .menu.pat.m add separator
        .menu.pat.m add command -lab $text(beenden) -com "schluss 1" \
                                -und 0 -acc $text(acc,beenden)
        menubutton .menu.datei -text $text(datei) -menu .menu.datei.m -underline 1
        menu .menu.datei.m
        .menu.datei.m add command -lab $text(spielzustand) -state disabled
        .menu.datei.m add command -lab "$text(laden)..." \
                                -com "laden" -acc $text(acc,laden) -und 0
        .menu.datei.m add command -lab "$text(speichern)..." \
                                -com speichern -acc $text(acc,speichern) -und 0
        .menu.datei.m add command -lab "$text(highScore)..." \
                                -com "highScore 1" -acc $text(acc,highScore) -und 0
        .menu.datei.m add separator
        .menu.datei.m add command -lab "$text(regelnEditieren)..." \
                                -com "regelEdit" -und 0
        .menu.datei.m add separator
        .menu.datei.m add command -lab $text(sprache) -state disabled
        foreach sprachDatei [glob -nocomplain [fullPath $patiencePath text *]] {
          set aktSprache  [file tail $sprachDatei]
          set aktSprache  [string tolower $aktSprache]
          set ersterGross [string toupper [string index $aktSprache 0]]
          set restKlein   [string range $aktSprache 1 end]
          .menu.datei.m add radio -lab $ersterGross$restKlein \
                        -var sprache -val $aktSprache         \
                        -command "setzSprache $sprachDatei"
        }
        menubutton .menu.hilfe -text $text(hilfe) -menu .menu.hilfe.m -underline 1
        menu .menu.hilfe.m
        .menu.hilfe.m add command -label "$text(erstellt)..." -com erstelltVon -und 0
        .menu.hilfe.m add separator
        foreach helpTopic {einleitung regeln kommandos streit
                           regeledit  installation wuerdigungen} {
          .menu.hilfe.m add command -label $text(title,$helpTopic) \
                 -com "ersteHilfe $text(file,$helpTopic)"
        }
        .menu.hilfe.m add separator
        .menu.hilfe.m add check -lab $text(netscape) -variable mitNetscape -und 4
        menubutton .menu.schmu -text $text(unterstuetzung) -menu .menu.schmu.m -und 1
        if {$streitArt($patArt)} {
          .menu.schmu configure -state disabled
        } 

        menu .menu.schmu.m
        .menu.schmu.m add command -lab $text(vorschlag) \
                                  -com "vorschlag 0" -acc $text(acc,vorschlag)
        .menu.schmu.m add command -lab $text(ablegen) \
                                  -com "ablegen" -acc $text(acc,ablegen)
        .menu.schmu.m add sep
        .menu.schmu.m add command -lab $text(zugZurueck) -com "zugZurueck"  \
                                        -acc $text(acc,zugZurueck)  -state disabled
        .menu.schmu.m add command -lab $text(zumAnfang) -com "zumAnfang" \
                                                        -state disabled
        .menu.schmu.m add sep
        .menu.schmu.m add check   -lab $text(merken)    -com merken -var gemerkt \
                                                     -state disabled
        .menu.schmu.m add command -lab $text(zurMarke)  -com "zurMarke" \
                                                        -state disabled
        .menu.schmu.m add sep
        .menu.schmu.m add check   -lab $text(veraendern)  -var veraendern
        menubutton .menu.karten -text $text(karten) -menu .menu.karten.m -und 1
        menu .menu.karten.m
        .menu.karten.m add command -lab $text(geben) \
                                   -command "geben hand" -acc $text(acc,geben)
        .menu.karten.m add command -lab $text(noch)0$text(kartenUebrig) \
                                                         -state disabled
        .menu.karten.m add sep
        .menu.karten.m add command -lab $text(kartenfonts) -state disabled
        foreach font [array names kartenFonts] {
          if {$tk_version >= 4.0 || ![info exists kartenFont($font,photo)]} {
            if {[info exists kartenFont($font,name-$sprache)]} {
              set fontLabel $kartenFont($font,name-$sprache)
            } else {
              set fontLabel font
            }
            .menu.karten.m add radio -label $fontLabel -var aktFont -val $font \
                                     -command {setBitPath $aktFont}
          }
        }
        .menu.karten.m add sep
        .menu.karten.m add command -lab $text(mitComics)     -state disabled
        .menu.karten.m add check   -lab $text(verteilen)     -var mitMischComic
        .menu.karten.m add check   -lab $text(kartenbewegen) -var mitBewegComic
        .menu.karten.m add command -lab $text(schatten:)$schatten... -com "schatten"
        global tk_version

        pack .menu.pat .menu.datei .menu.schmu .menu.karten -side left
        pack .menu.hilfe -side right
        bind . <Any-Enter> {focus $flaeche}
        if {$tk_version < 4.0} {
          tk_bindForTraversal $flaeche
          tk_menuBar .menu .menu.pat .menu.datei .menu.schmu .menu.karten .menu.hilfe
        }
        trace variable zugNr        w chkZugNr
      }
    proc chkZugNr {name1 name2 op} {
      global zugNr marke gemerkt streitArt patArt text

      if {!$streitArt($patArt)} {
        confMenueEntry .menu.schmu.m $text(zugZurueck) \
            -state [expr {$zugNr > 0 ? "normal" : "disabled"}]
        confMenueEntry .menu.schmu.m $text(zumAnfang) \
            -state [expr {$zugNr > 0 ? "normal" : "disabled"}]
        if {$zugNr<$marke} {set gemerkt 0}
        confMenueEntry .menu.schmu.m $text(zurMarke) \
            -state [expr {$gemerkt && $zugNr > $marke ? "normal" : "disabled"}]
        confMenueEntry .menu.schmu.m $text(merken) \
            -state [expr {$zugNr > 0 ? "normal" : "disabled"}]
      }
    }
    proc chkKartenAnz {name1 name2 op} {
      upvar #0 $name1 fuelle
      global text streitArt patArt flaeche auchBild

      if {$auchBild} {
        $flaeche itemconfigure \
            [expr {$name2 == "hand2" ? "kartenAnz2" : "kartenAnz"}] \
            -text $fuelle($name2)

        if {$name2 == ($streitArt($patArt) ? "hand2" : "hand")} {
          confMenueEntry .menu.karten.m "$text(noch)*$text(kartenUebrig)" \
            -label $text(noch)$fuelle($name2)$text(kartenUebrig)
        }
      }
    }
    proc confMenueEntry {m index args} {
      set tearOffs [info commands .tearoff*]
      if {[llength $tearOffs]} {
        set m "$m $tearOffs"
      }
      foreach menu $m {
        catch {eval $menu entryconfigure [list $index] $args}
      }
    }
    proc unpostMenueEntry {m index} {
      set tearOffs [info commands .tearoff*]
      if {[llength $tearOffs]} {
        set m "$m $tearOffs"
      }
      foreach menu $m {
        if {![catch {$menu index $index}]} {
          $menu unpost
        }
      }
    }
    proc aktiviereKritischeKommandos {an} {
      global flaeche text streitArt patArt

      if {![string compare $an ein]} {
        if {!$streitArt($patArt)} {
          confMenueEntry .menu.pat.m  $text(demo)      -state normal
        }
        confMenueEntry .menu.schmu.m  $text(vorschlag) -state normal
        confMenueEntry .menu.karten.m $text(geben)     -state normal
      } else {
        confMenueEntry .menu.pat.m    $text(demo)      -state disabled
        confMenueEntry .menu.schmu.m  $text(vorschlag) -state disabled
        confMenueEntry .menu.karten.m $text(geben)     -state disabled

        bind $flaeche <$text(acc,demo)> ""
        bind $flaeche <$text(acc,neuesSpiel)> ""
        bind $flaeche <$text(acc,vorschlag)> ""
        bind $flaeche <$text(acc,geben)> ""
      }
    }
    proc aktiviereGeben {hand an} {
      global flaeche text demoAktiv

      if {![string compare $an ein]} {
        if {!$demoAktiv} {
          $flaeche bind $hand <1> "geben $hand"
          bind $flaeche $text(acc,geben) "geben $hand"
        }
        confMenueEntry .menu.karten.m $text(geben) -state normal
      } else {
        nichtVerschiebbar $hand
        bind $flaeche $text(acc,geben) ""
        confMenueEntry .menu.karten.m $text(geben) -state disabled
      }
    }
    proc aktiviereMenuepunkteFuerStreit {} {
      global text du streitArt patArt veraendern streitbar

      if {$streitArt($patArt)} {
        if {[string compare [lindex [.menu.schmu configure -state] 4] disabled]} {
          .menu.schmu configure -state disabled
        }
        unpostMenueEntry .menu.schmu.m $text(vorschlag)
        set veraendern 0
      } else {
        if {[string compare [lindex [.menu.schmu configure -state] 4] normal]} {
          .menu.schmu configure -state normal
        }
      }
      confMenueEntry .menu.pat.m $text(streitMit)* -label "$text(streitMit)$du"

      foreach punkt {kontaktAufnahme kontaktPruefen kontaktAbgleich} {
        confMenueEntry .menu.pat.m $text($punkt)... -state disabled
      }
      if {$streitArt($patArt) && $streitbar} {
        if {![string compare $du $text(computer)]} {
          confMenueEntry .menu.pat.m $text(kontaktAufnahme)... -state normal
        } else {
          confMenueEntry .menu.pat.m $text(kontaktPruefen)...  -state normal
          confMenueEntry .menu.pat.m $text(kontaktAbgleich)... -state normal
        }
      }
      if {[info tclversion] < 7.5} {
        confMenueEntry .menu.pat.m $text(streitBereit) -state disabled
      }
    }
    proc istGebenAktiv {} {
      global text

      set status [lindex [.menu.karten.m entryconfigure $text(geben) -state] 4]
      return [expr {$status == "normal"}]
    }
    proc menueHilfe {was} {
      global alteMenueLeiste text

      switch $was {
        aus {
          catch {destroy .menu.endtext}
          eval $alteMenueLeiste
        }
        default {
          if {![info exists alteMenueLeiste]} {
            set alteMenueLeiste ""
            global tk_version

            set new [expr {$tk_version >= 4.0 ? "" : "new"}]
            foreach menuePunkt [pack slaves .menu] {
              set aktiv [lindex [$menuePunkt configure -state] 4]
              append alteMenueLeiste "
                     pack $menuePunkt [pack ${new}info $menuePunkt]
                     $menuePunkt configure -state $aktiv"
            }
          }
          foreach menuePunkt [pack slaves .menu] {
            if {![catch {$menuePunkt configure -state disabled}]} {
              pack forget $menuePunkt
            }
          }
          catch {label .menu.endtext -text $text(${was}Hilfe)}
          place .menu.endtext -anchor w -rely 0.5
        }
      }
      update
    }
    proc speichern {} {
      global zustDateiName text

      if [holDateiName $text(speicherAuswahl) zustDateiName "" \
                       "*.pat" "spiel.pat"] {
        schrZustand zustand 0

        set aus [open $zustDateiName "w"]
        foreach zeile $zustand {
          puts $aus $zeile
        }
        close $aus
      }
    }
    proc laden {} {
      global zustDateiName text streitArt patArt du zugNr \
             demoAktiv kommandoNachDemo

      if [holDateiName $text(ladeAuswahl) zustDateiName "*" \
                       "*.pat" "spiel.pat"] {
        if {$streitArt($patArt) && [string compare $du $text(computer)]} {
          if {![kontaktBeenden 0]} return
          if {$demoAktiv} {
            set demoAktiv        0
            set kommandoNachDemo "liesZustand $zustDateiName datei"
            return
          }
        }
        liesZustand $zustDateiName datei
      }
    }
    proc holDateiName {text dateiName magic ext defaultName} {
      upvar 1 $dateiName name

      if {![info exist name] || ![string length $name]} {
        set dir   [pwd]
        set datei $defaultName
      } else {
        set dir   [file dirname $name]
        set datei [file tail    $name]
      }
      set neuName [fileSelect .fs $text $datei $dir $ext]
      if {![string length $neuName]} {return 0}

      switch -regexp $magic {
        . {
          if {![file readable $neuName]} {
            nichtLesbar $neuName
            return 0
          }
        }
        default {
          if {[file exist $neuName]} {
            set allesKlar [expr {[file writable $neuName] &&
                                 ![file isdirectory $neuName]}]
          } else {
            set zielDirectory [file dirname $neuName]
            set allesKlar [expr {[file writable $zielDirectory] &&
                                 [file isdirectory $zielDirectory]}]
          }
          if {!$allesKlar} {
            nichtSchreibbar $neuName
            return 0
          }
        }
      }
      set name $neuName
      return 1
    }
    proc setzSprache {sprachDatei} {
      global streitArt patArt sprache text du

      if {[string compare $du $text(computer)]} {
        set gegnerName $du
      }

      if [catch {source $sprachDatei}] return

      if {![info exists gegnerName]} {
        set gegnerName $text(computer)
      }

      destroy .menu
      setzBenutzernamen $gegnerName
      mkMenue
      aktiviereBindings
      zeigHinweis
      highScore   0
      aktiviereMenuepunkteFuerStreit
      global patArt zugNr fuelle
      foreach var {patArt zugNr fuelle(hand) fuelle(hand2)} {
        catch {set $var [set $var]}
      }

      regelEdit 1
    }
    proc highScore {neu {neuHighScore ""}} {
      global text patArt patName streitArt sprache tk_version highScoreStatus

      set w .highScore
      set boxen {nr spieler zuege vorschlag karten du zeit}

      if {![winfo exist $w]} {
        if {!$neu} return
        set highScoreStatus(sprache)   ""
        set highScoreStatus(patArt)    ""
        set highScoreStatus(streitArt) ""

        toplevel $w
        wm iconname $w $text(highScore)

        button $w.ok -text $text(okay) -command "destroy $w"
        pack   $w.ok -side bottom -fill x

        label $w.meldung -text ""
        pack  $w.meldung -side bottom -anchor w

        option add *highScore*highlightThickness 0 widgetDefault
        option add *highScore*font \
            -Adobe-times-medium-r-normal--*-140*   widgetDefault

        frame $w.h
        pack  $w.h -anchor w -fill x
        frame $w.l
        pack  $w.l -expand 1 -fill both

        foreach box {{nr         6} {spieler 20} {zuege  10}
                     {vorschlag 10} {du      20} {karten 10} {zeit 30}} {
          set name  [lindex $box 0]
          set width [lindex $box 1]

          text $w.h.$name -relief raised -bd 2 -width $width -height 1
          pack $w.h.$name -side left -fill x -expand 1
          text $w.l.$name -relief raised -bd 2 -width $width -height 10 \
                          -setgrid 1 -yscrollcommand "$w.l.s set"
          pack $w.l.$name -side left -expand 1 -fill both

          $w.l.$name tag configure neueZeile  -foreground red
          if {$tk_version >= 4.0} {
            $w.l.$name tag configure zentrieren -justify center
          }
        }

        scrollbar $w.l.s -orient vertical -command [list multiYview $w.l $boxen]
        pack      $w.l.s -side right -fill y

        if {$tk_version >= 4.0} {
          set scrollWidth [expr {[$w.l.s cget -width] \
                              +2*[$w.l.s cget -bd]    \
                              +2*[$w.l.s cget -highlightthickness]}]
        } else {
          set scrollWidth [expr {[lindex [$w.l.s configure -width] 4] \
                              +2*[lindex [$w.l.s configure -bd]    4]}]
        }
        frame $w.h.pad -width $scrollWidth
        pack  $w.h.pad -side right

        foreach box $boxen {
          $w.h.$box tag configure ueberschrift \
                    -font -Adobe-times-bold-r-normal--*-140*
          if {$tk_version >= 4.0} {
            $w.h.$box tag configure ueberschrift -justify center
          }
        }
      } else {
        if {$neu} {
          if {![string compare [wm state $w] iconic]} {wm deiconify $w}
          raise $w
        }
        foreach box $boxen {
          $w.l.$box configure -state normal
          $w.l.$box delete 1.0 end
        }
      }

      if {[string compare $patArt  $highScoreStatus(patArt)] ||
          [string compare $sprache $highScoreStatus(sprache)]} {
        wm title $w "$text(highScore)/$patName($patArt,$sprache)"

        foreach box $boxen {
          $w.h.$box configure -state normal
          $w.h.$box delete 1.0 end
          $w.h.$box insert 1.0 $text(bst:$box)
          $w.h.$box tag add ueberschrift 1.0 1.end
          $w.h.$box configure -state disabled
        }

        if {[string compare $streitArt($patArt) $highScoreStatus(streitArt)]} {
          pack forget $w.h.zuege $w.h.vorschlag $w.h.karten $w.h.du \
                      $w.l.zuege $w.l.vorschlag $w.l.karten $w.l.du
          if {$streitArt($patArt)} {
            pack $w.h.karten $w.h.du -side left -after $w.h.spieler \
                                     -fill x    -expand 1
            pack $w.l.karten $w.l.du -side left -after $w.l.spieler \
                                     -fill both -expand 1
          } else {
            pack $w.h.zuege $w.h.vorschlag \
                    -side left -after $w.h.spieler -fill x    -expand 1
            pack $w.l.zuege $w.l.vorschlag \
                    -side left -after $w.l.spieler -fill both -expand 1
          }
        }

        set highScoreStatus(sprache)   $sprache
        set highScoreStatus(patArt)    $patArt
        set highScoreStatus(streitArt) $streitArt($patArt)
      }

      if {[string compare $neuHighScore ""]} {
        set neuRang   [lindex $neuHighScore 0]
        set highScore [lindex $neuHighScore 1]
        set meldung   [lindex $neuHighScore 2]
      } else {
        set neuRang   -1
        set highScore [gibHighScore $patArt]
        set meldung   "$text(highScore)$text(fuer)$patName($patArt,$sprache)"
      }
      $w.meldung configure -text $meldung

      set rang 0
      foreach zeile $highScore {
        $w.l.nr        insert end "[incr rang]\n"
        $w.l.spieler   insert end "[lindex $zeile 0]\n"
        $w.l.zeit      insert end "[lindex $zeile 3]\n"
        if {$streitArt($patArt)} {
          $w.l.karten insert end "[lindex $zeile 1]\n"
          $w.l.du     insert end "[lindex $zeile 2]\n"
        } else {
          $w.l.zuege     insert end "[lindex $zeile 1]\n"
          $w.l.vorschlag insert end "[lindex $zeile 2]\n"
        }
      }
      if {$neuRang > 0} {
        foreach box $boxen {
          $w.l.$box tag add neueZeile $neuRang.0 $neuRang.end
        }
        $w.l.[lindex $boxen 0] see $neuRang.0
      }

      foreach box $boxen {
        if {$tk_version >= 4.0} {
          switch $box nr - zuege - vorschlag - karten {
            $w.l.$box tag add zentrieren 1.0 end 
          }
        }
        $w.l.$box configure -state disabled
      }
    }
    proc multiYview {w lists args} {
      foreach l $lists {
        eval {$w.$l yview} $args
      }
    }
    proc optionReadfile {filename prioritaet} {
        global tcl_platform

        switch $tcl_platform(platform) {
          unix {
            option readfile $filename $prioritaet
          }
          default {
            option add *Menubutton*disabledForeground grey50 widgetDefault

            set appDefRE "^\[ \t]*(\[^ \t]*atience\[^ \t:]*)\[ \t]*:\[ \t]*(\[^ \t]*)\[ \t]*$"
            if {![catch {open $filename r} file]} {
              while {[gets $file line] >= 0} {
                regsub "(^\[^:]*!).*$" $line "\1" line
                if {[regexp $appDefRE $line match resource value]} {
                  option add $resource $value $prioritaet
                 }
              }
              close $file
            }
          }
        }
      }
      proc mkFlaeche {} {
        global flaeche roteFarbe weisseFarbe graueFarbe warteZeit mitNetscape \
               mitMischComic mitBewegComic tk_version streitbar schatten

        wm withdraw .
        set flaeche [canvas .patience -closeenough 0.0]
        if {$tk_version >= 4.0} {
          $flaeche configure -highlightthickness 0
        }
        global patiencePath env
        catch {option readfile $patiencePath/patience.ad startupFile}
        catch {option readfile $env(HOME)/.Xdefaults     userDefault}
        #optionReadfile $patiencePath/patience.ad startupFile
        #optionReadfile $env(HOME)/.Xdefaults     userDefault
        option add *patience.warteZeit     10                  widgetDefault
        option add *patience.rot           IndianRed4          widgetDefault
        option add *patience.grau          gray80              widgetDefault
        option add *patience.weiss         gray95              widgetDefault
        option add *patience.sprPath       $patiencePath/spiel widgetDefault
        option add *patience.mitMischComic True                widgetDefault
        option add *patience.mitBewegComic True                widgetDefault
        option add *patience.schatten      3                   widgetDefault
        option add *patience.mitStreitbar  False               widgetDefault
        option add *patience.kartenFont    normal              widgetDefault
        option add *patience.sprache       english             widgetDefault
        option add *patience.mitNetscape   False               widgetDefault
        option add *patience.daemonPort    4567                widgetDefault
        liesTexte

        global kartenFont kartenFonts patiencePath aktFont

        foreach satz [glob -nocomplain [fullPath $patiencePath bitmaps * dim.tcl]] {
          source $satz
          set kartenFonts($kartenSatz) $kartenSatz
        }
        set aktFont [option get $flaeche kartenFont KartenFont]
        if {![info exists kartenFonts($aktFont)]} {
          set aktFont normal
          if {![info exists kartenFonts($aktFont)]} {
            global text

            keineKarten
            exit 1
          }
        }

        set warteZeit [option get $flaeche warteZeit WarteZeit]
        if [catch {set warteZeit [expr int($warteZeit)]}] {
          set warteZeit 10
        }
        set schatten  [option get $flaeche schatten Schatten]
        if {[catch {set schatten [expr int($schatten)]}] || $schatten < 0} {
          set schatten 3
        }

        set roteFarbe   [option get $flaeche rot   Foreground]
        set graueFarbe  [option get $flaeche grau  Background]
        set weisseFarbe [option get $flaeche weiss Background]

        set mitMischComic [string match {[Tt]*} \
                               [option get $flaeche mitMischComic MitComic]]
        set mitBewegComic [string match {[Tt]*} \
                               [option get $flaeche mitBewegComic MitComic]]
        streitbar [string match {[Tt]*} \
                               [option get $flaeche mitStreitbar  mitStreitbar]]
        set mitNetscape [string match {[Tt]*} \
                               [option get $flaeche mitNetscape MitNetscape]]

        pack $flaeche -side bottom -expand 1 -fill both
      }
      proc fuellFlaeche {} {
        global patiencePath patName zugNr flaeche patArt tk_version

        trace variable patArt w setzFenstertitel
        setzFenstertitel patArt "" w

        trace variable patArt w setzStrafX
        bind $flaeche <Configure> "setBitPath"
        setBitPath <>

        update
        wm minsize . [winfo width .] [winfo height .]
      }
      proc setzFenstertitel {art leer op} {
        global patienceVersion patName text sprache
        upvar #0 $art patArt

        if {[info exists patName($patArt,$sprache)]} {
          set name $patName($patArt,$sprache)
        } else {
          set name $patArt
        }
        wm title . "$text(patience) $patienceVersion / $name"
      }
      if {$tk_version >= 4.0} {
        proc erzeugeKarten {} {
          global aktFont bitPath weisseFarbe roteFarbe graueFarbe \
                 kartenFont erzeugterFont flaeche

          if {[info exists erzeugterFont]
              && ![string compare $erzeugterFont $aktFont]} return

          set liste {ruecken h w sch}
          foreach farbe {c h p k} {
            foreach wert {s a 2 3 4 5 6 7 8 9 0 b d k} {
              lappend liste $wert$farbe
            }
          }

          if {[info exists kartenFont($aktFont,photo)]} {
            foreach karte $liste {
              image create photo $karte -file $bitPath/$karte
            }
          } else {
            set mitFarbe [expr {![info exists kartenFont($aktFont,sw)]}]
            set mitMaske [info exists kartenFont($aktFont,maske)]
            if {$mitMaske} {
              set maske $kartenFont($aktFont,maske)
            }

            foreach karte $liste {
              image create bitmap $karte -file $bitPath/$karte
              if {$mitMaske} {
                $karte configure -maskfile $bitPath/$maske
              }
              if {[string match {?[ch]} $karte]
                  && ($mitFarbe || [string index $karte 0] != "s")} {
                $karte configure -foreground $roteFarbe
              }
              if {$mitFarbe && [string match {[sh]*} $karte]} {
                $karte configure -background $graueFarbe
              } else {
                $karte configure -background $weisseFarbe
              }
            }
          }
          set erzeugterFont $aktFont
        }
      }
      proc stelleKarteDar {flaeche neu karte tag x y} {
        global tk_version

        if {$tk_version >= 4.0} {
          if {![string compare $neu neu]} {
            $flaeche create image $x $y -anchor nw -tag $tag
          }
          $flaeche itemconfigure $tag -image $karte
        } else {
          global weisseFarbe roteFarbe graueFarbe bitPath kartenFont aktFont

          if {![string compare $neu neu]} {
            $flaeche create bitmap $x $y -background $weisseFarbe \
                                         -anchor nw -tag $tag
          }
          $flaeche itemconfigure $tag -bitmap @$bitPath/$karte
          set mitFarbe [expr {![info exists kartenFont($aktFont,sw)]}]
          if {[string match {?[ch]} $karte]
              && ($mitFarbe || [string index $karte 0] != "s")} {
            $flaeche itemconfigure $tag -foreground $roteFarbe
          } else {
            $flaeche itemconfigure $tag -foreground black
          }
          if {$mitFarbe && [string match {[sh]*} $karte]} {
            $flaeche itemconfigure $tag -background $graueFarbe
          } else {
            $flaeche itemconfigure $tag -background $weisseFarbe
          }
        }
      }
    proc aktiviereBindings {{command {}}} {
      global flaeche streitArt patArt text spezialKommando

      set spezialKommando $command
      if {![string length $command]} {
        foreach event [list Key Button] {
          bind $flaeche <$event> ""
        }
        foreach stapel {hand hand2} {
          $flaeche bind $stapel <1> "geben $stapel"
        }
        foreach klick {<3> <Double-1>} {
          bind $flaeche $klick "tuWasGutes $klick %x %y"
        }
        bind $flaeche <$text(acc,speichern)> "if {%s==0||%s==8} {speichern}"
        bind $flaeche <$text(acc,laden)>     "if {%s==0||%s==8} {laden}"
        bind $flaeche <Control-Key>          "vorschlagDebug %K"
        if {!$streitArt($patArt)} {
          bind $flaeche <$text(acc,ablegen)>    "if {%s==0||%s==8} {ablegen}"
          bind $flaeche <$text(acc,vorschlag)>  "if {%s==0||%s==8} {vorschlag 0}"
          bind $flaeche <$text(acc,zugZurueck)> "if {%s==0||%s==8} {zugZurueck}"
          bind $flaeche <$text(acc,demo)>       "if {%s==0||%s==8} {demo}"
          bind $flaeche <$text(acc,geben)>      "if {%s==0||%s==8} {geben hand}"
        } else {
          bind $flaeche <$text(acc,geben)>      "if {%s==0||%s==8} {geben hand2}"
        }
      } else {
        foreach stapel {hand hand2} {
          $flaeche bind $stapel <1> $command
          $flaeche bind $stapel <3> $command
        }
        if {[string index $command 0] == "+"} {
          set command [string range $command 1 end]
          set addBinding "+"
        } else {
          set addBinding ""
        }
        foreach event [list $text(acc,geben)     $text(acc,ablegen)    \
                            $text(acc,vorschlag) $text(acc,demo)       \
                            $text(acc,speichern) $text(acc,zugZurueck) \
                            $text(acc,laden)     Key Button] {
          bind $flaeche <$event> \
                    "${addBinding}if {\[string compare %K Control_L]} {$command}"
        }
      }
      bind $flaeche <$text(acc,highScore)>  "if {%s==0||%s==8} {highScore 1}"
      bind $flaeche <$text(acc,beenden)>    "if {%s==0||%s==8} {schluss 1}"
      bind $flaeche <$text(acc,neuesSpiel)> "if {%s==0||%s==8} {starteSpiel 1}"
      bind $flaeche <Control-l>             "bildNeu"
    }
    proc setBitPath {{neuFont ""}} {
      global bitPath patiencePath kartenHoehe stapelBreite fuelle \
             pos gesStockHoehe bildBreite bildHoehe flaeche       \
             stockHoehe exist tk_version streitArt stapelAnz      \
             kartenSatz patArt kartenFont aktFont stapel erzeugterFont

      set stapelBreite  $kartenFont($aktFont,breite)
      set kartenHoehe   $kartenFont($aktFont,hoehe)
      set stockHoehe    [expr {$kartenHoehe*2/3}]
      set gesStockHoehe [expr {$stockHoehe*3+$kartenHoehe+2}]

      set minStapel [expr {$stapelAnz($patArt)+$kartenSatz($patArt)
                                              +($exist($patArt,arbeit) > 0)}]
      if {$minStapel < 10} {set minStapel 10}
      set minBreite     [expr {$stapelBreite*$minStapel}] 
      set minHoehe      [expr {$gesStockHoehe+$kartenHoehe+30}]

      set bitPath "$patiencePath/bitmaps/$aktFont"
      if {$tk_version >= 4.0 && ![info exists erzeugterFont]} {
        erzeugeKarten
      }

      set derzeitigeBreite [winfo width $flaeche]
      set derzeitigeHoehe  [winfo height $flaeche]
      if {![string compare $neuFont "<>"]
           && $minBreite <= $derzeitigeBreite
           && $minHoehe  <= $derzeitigeHoehe} {
        set neuFont ""
      }
      if {[string length $neuFont]} {
        wm geometry . {}
        $flaeche configure -width $minBreite -height $minHoehe
        update
        if {$minHoehe != $derzeitigeHoehe || $minBreite != $derzeitigeBreite} {
          return
        }
      }

      set bildBreite [winfo width  $flaeche]
      set bildHoehe  [winfo height $flaeche]
      set pos(x,hand)      [expr $bildBreite-$stapelBreite+4]
      set pos(y,hand)      [expr $bildHoehe-$kartenHoehe]
      set pos(x,hand2)     0
      set pos(y,hand2)     $pos(y,hand)
      set pos(x,talon)     [expr $bildBreite-2*$stapelBreite]
      set pos(y,talon)     $pos(y,hand)
      set pos(x,talon2)    [expr $stapelBreite+2]
      set pos(y,talon2)    $pos(y,hand)
      set pos(x,schachUhr) [expr $bildBreite/2]
      set pos(y,schachUhr) $bildHoehe
      set pos(y,strafe)    $pos(y,hand)
      set pos(x,strafe2)   [expr 2*$stapelBreite+2]
      set pos(y,strafe2)   $pos(y,hand)
      setzStrafX patArt "" w

      if {$bildBreite < $minBreite || $bildHoehe < $minHoehe} {
        # Notfall: Bildschirmgroesse reicht fuer Font nicht aus; deshalb jetzt
        # auf kleinen Font umschalten, und sich selbst rekursiv aufrufen.
        set aktFont normal
        setBitPath $aktFont
      } elseif {[info exists stapel]} {
        # Bild mit dem aktuellen Spiel neu aufbauen.
        bildNeu
      }
    }
    proc setzStrafX {art leer op} {
      global pos bildBreite stapelBreite streitArt
      upvar #0 $art patArt

      if {$streitArt($patArt)} {
        set pos(x,strafe) [expr {$bildBreite-3*$stapelBreite}]
      } else {
        set pos(x,strafe) 0
      }
    }
    proc spielXY {spielX spielY absX absY} {
      upvar $spielX relX $spielY relY
      global bildBreite bildHoehe flaeche

      set relX [$flaeche canvasx $absX]
      set relY [$flaeche canvasy $absY]
      if {$relX<0} {set relX 0} elseif {$relX>$bildBreite} {set relX $bildBreite}
      if {$relY<0} {set relY 0} elseif {$relY>$bildHoehe}  {set relY $bildHoehe}
    }
    proc stapelX {stapel} {
      global stapelBreite bildBreite pos kartenSatz patArt

      switch -glob $stapel {
        ?       {return [expr {$stapel*$stapelBreite+2}]}
        ?,?     {return [expr {$bildBreite 
                                 - $stapelBreite*([string index $stapel 0]+1)}]}
        arbeit? {return [expr {$bildBreite
                                 - $stapelBreite*($kartenSatz($patArt)+1)}]}
        default {return $pos(x,$stapel)}
      }
    }
    proc stapelY {stapel {aktY 0}} {
      global stockHoehe bildHoehe kartenHoehe fuelle exist streitArt patArt pos

      switch -glob $stapel {
        ? {
          if {$aktY <= 0} {return 2}

          set verfuegbar [expr $bildHoehe-10]
          switch $stapel 0 - 1 - 2 {
            if {$streitArt($patArt) || $exist($patArt,strafe) > 0} {
              incr verfuegbar [expr -2*$kartenHoehe]
            }
          }
          set maxHoehe [expr {double($bildHoehe)/20}]
          if {$fuelle($stapel) > 1} {
            set hoeheProKarte [expr {double($verfuegbar)/$fuelle($stapel)}]
            if {$hoeheProKarte > $maxHoehe} {set hoeheProKarte $maxHoehe}
          } else {
            set hoeheProKarte $maxHoehe
          }

          return [expr {$aktY*$hoeheProKarte+2}]
        }
        ?,?     {return [expr {2+$stockHoehe*[farbRang $stapel]}]}
        arbeit? {return [expr {2+$stockHoehe*[string index $stapel 6]}]}
        default {return $pos(y,$stapel)}
      }
    }
    proc findeStapel {neuX neuY} {
      global stapelAnz kartenSatz streitArt patArt fuelle pos exist \
          stapelBreite stockHoehe gesStockHoehe bildBreite kartenHoehe
      
      spielXY x y $neuX $neuY
      
      foreach sonderStapel {talon talon2 strafe strafe2} {
        if {[info exists fuelle($sonderStapel)] && $fuelle($sonderStapel) > 0    \
            && $x>=$pos(x,$sonderStapel)&&$x<$pos(x,$sonderStapel)+$stapelBreite \
            && $y>=$pos(y,$sonderStapel)&&$y<$pos(y,$sonderStapel)+$kartenHoehe} {
          return $sonderStapel
        }
      }
      foreach hand {hand hand2} {
        if {$x>=$pos(x,$hand)&&$x<$pos(x,$hand)+$stapelBreite && \
            $y>=$pos(y,$hand)&&$y<$pos(y,$hand)+$kartenHoehe} {
          if {![string compare $hand hand] || $streitArt($patArt)} {return $hand}
        }
      }
      if {$x < $stapelAnz($patArt)*$stapelBreite} {
        return [expr int($x/$stapelBreite)]
      }
      if {$x > $bildBreite-$kartenSatz($patArt)*$stapelBreite &&
          $y < $gesStockHoehe} {
        if {$y >= $stockHoehe*4} {set y [expr $stockHoehe*4]}
        set aktSatz  [expr int (($bildBreite-$x)/$stapelBreite)]
        set aktFarbe [kartenFarbe [expr int (($y-2)/$stockHoehe)]]
        return $aktSatz,$aktFarbe
      }
      if {$exist($patArt,arbeit)
          && $x > $bildBreite-(1+$kartenSatz($patArt))*$stapelBreite
          && $y < $kartenHoehe+($exist($patArt,arbeit)-1)*$stockHoehe} {
        if {$y >= $stockHoehe*$exist($patArt,arbeit)} {
          set y [expr $stockHoehe*$exist($patArt,arbeit)]
        }
        return arbeit[expr int (($y-2)/$stockHoehe)]
      }
      return ""
    }
    proc umdrehbar {karte stapel} {
      global flaeche

      $flaeche bind $karte <1> "drehUm $karte $stapel 1"
    }
    proc verschiebbar {karte stapel stapY} {
      global flaeche

      $flaeche bind $karte <1> "startEinzelKarte $karte %x %y $stapel $stapY"
    }
    proc teilStapelBar {karte anwStapel untenY} {
      global flaeche

      $flaeche bind $karte <1> "startTeilStapel $anwStapel $untenY %x %y"
    }
    proc nichtVerschiebbar {karte} {
      global flaeche

      $flaeche bind $karte <1> ""
      nichtBewegbar $karte
    }
    proc nichtBewegbar {karte} {
      global flaeche

      $flaeche bind $karte <B1-Motion>           ""
      $flaeche bind $karte <Any-ButtonRelease-1> ""
    }
    proc kurzePause {} {
      after 1 {set kurzePauseZumDruecken 1}
      tkwait variable kurzePauseZumDruecken
    }
    proc pruefStapelFuelle {aktStapel} {
      global stapel fuelle flaeche

      set oben [expr {$fuelle($aktStapel)-1}]
      if {$oben < 1 || [lindex [$flaeche coords $stapel($aktStapel,$oben)] 1]
                         == [stapelY $aktStapel $oben]} return

      set aktX [stapelX $aktStapel]
      for {set aktI 0} {$aktI < $fuelle($aktStapel)} {incr aktI} {
        $flaeche coords $stapel($aktStapel,$aktI) \
                        $aktX [stapelY $aktStapel $aktI]
      }
    }
    proc maleLeereStoecke {} {
      global kartenSatz patArt flaeche streitArt exist

      for {set aktSatz 0} {$aktSatz < $kartenSatz($patArt)} {incr aktSatz} {
        set letzter ""
        foreach aktFarbe {c h p k} {
          set tag s$aktFarbe$aktSatz
          stelleKarteDar $flaeche neu s$aktFarbe $tag \
                    [stapelX $aktSatz,$aktFarbe] [stapelY $aktSatz,$aktFarbe]
          $flaeche lower $tag
          if {[string length $letzter]} {
            $flaeche raise $tag $letzter
          }
          set letzter $tag
        }
      }
      set letzter ""
      for {set aktArb 0} {$aktArb < $exist($patArt,arbeit)} {incr aktArb} {
        set tag w$aktArb
        stelleKarteDar $flaeche neu w $tag \
                    [stapelX arbeit$aktArb] [stapelY arbeit$aktArb]
        $flaeche lower $tag
        if {[string length $letzter]} {
          $flaeche raise $tag $letzter
        }
        set letzter $tag
      }
    }
    proc maleZugbestaetigungsknopf {} {
      global flaeche aktiverSpieler weisseFarbe roteFarbe text ich tk_version

      set x [stapelX schachUhr]
      set y [stapelY schachUhr]
      $flaeche create rectangle [expr {$x-150/2}] [expr {$y-50}] \
                                [expr {$x+150/2}] $y             \
                    -fill white -outline white -tags uhrKnopf
      $flaeche create text $x [expr {$y-25}] -tags schachUhr -width 0
      set lr [expr {[stapelX schachUhr] - 75}]
      set ll [expr {$lr - 10}]
      set rl [expr {[stapelX schachUhr] + 75}]
      set rr [expr {$rl + 10}]
      set u  [stapelY schachUhr]
      set o  [expr {$u - 50}]
      set m  [expr {($o+$u)/2}]
      $flaeche create polygon $lr $u $lr $o $ll $m $lr $u -tags links
      $flaeche create polygon $rl $u $rl $o $rr $m $rl $u -tags rechts
      trace variable aktiverSpieler w chkAktiverSpieler
      chkAktiverSpieler aktiverSpieler "" w
    }
    proc chkAktiverSpieler {name1 name2 op} {
      global aktiverSpieler ich du flaeche roteFarbe

      $flaeche bind uhrKnopf <1> ""
      $flaeche itemconfigure schachUhr -fill grey64 -text $aktiverSpieler
      $flaeche itemconfigure links     -fill ""
      $flaeche itemconfigure rechts    -fill ""

      switch $aktiverSpieler "
        $du {
          $flaeche itemconfigure rechts    -fill $roteFarbe
        }
        $ich {
          $flaeche itemconfigure schachUhr -fill black
          $flaeche itemconfigure links     -fill $roteFarbe
          $flaeche bind uhrKnopf <1> streitZugBestaetigung
        }
      "
    }
    proc streitZugBestaetigung {} {
      global aktiverSpieler text flaeche roteFarbe du

      catch {global flaeche; $flaeche delete hinweis}
      set aktiverSpieler $du
      if {![string compare $du $text(computer)]} {
        demo
      } else {
        informiereStreitGegner SchachUhr
        aktiviereBindings "#"
      }
    }
    proc schaltSchachuhrAufMensch {mitBindings} {
      global flaeche aktiverSpieler ich talonAbraeumbar patArt stapel \
             fuelle verdeckt roteFarbe

      set aktiverSpieler $ich
      aktiviereGeben hand2 ein
      deaktiviereGegnerStapel
      
      if {!$talonAbraeumbar($patArt)} {
        foreach talon {talon talon2} {
          if {[info exists fuelle($talon)] && $fuelle($talon) > 0} {
            set verdeckt($talon) $fuelle($talon)
            nichtVerschiebbar $stapel($talon,[expr {$fuelle($talon)-1}])
          }
        }
      }
      zeigHinweis schachUhr
      if {$mitBindings} {
        aktiviereBindings
      }
    }
    proc zeigHinweis {{hinweisText ""}} {
      global hinweisGezeigt bildBreite bildHoehe text flaeche

      if {![string length $hinweisText]} {
        if {[string compare [$flaeche gettags hinweis] hinweis]} return
        set hinweisText $hinweisGezeigt(aktuell)
        catch {global flaeche; $flaeche delete hinweis}
      } elseif {[info exists hinweisGezeigt($hinweisText)]} {
        return
      }

      set farbe [lindex [$flaeche configure -background] 4]

      set grundlinie $bildHoehe
      if {![string compare [$flaeche gettags schachUhr] schachUhr]} {
        set grundlinie [expr {$grundlinie-50}]
      }
      $flaeche create rectangle \
            [expr {$bildBreite/2-100}] [expr {$grundlinie-60}] \
            [expr {$bildBreite/2+100}] [expr {$grundlinie}]    \
            -outline $farbe -fill $farbe -tags hinweis
      $flaeche create text [expr {$bildBreite/2}] [expr {$grundlinie-30}] \
            -text $text(hinweis:$hinweisText) -justify center -tags hinweis
      $flaeche bind hinweis <Button> \
            "catch {global flaeche; $flaeche delete hinweis}"

      set hinweisGezeigt(aktuell)      $hinweisText
      set hinweisGezeigt($hinweisText) 1
    }
    proc deaktiviereGegnerStapel {} {
      global fuelle stapel

      foreach sonderStapel {talon strafe} {
        if {[info exists fuelle($sonderStapel)] && $fuelle($sonderStapel) > 0} {
          set oben [expr {$fuelle($sonderStapel)-1}]
          nichtVerschiebbar $stapel($sonderStapel,$oben)
        }
      }
    }
    proc bildNeu {} {
      global stapel fuelle verdeckt bildBreite bildHoehe stapelBreite \
             patArt stapelAnz kartenSatz exist flaeche pos streitArt tk_version

      $flaeche delete all
      update
      if {$tk_version >= 4.0} {
        erzeugeKarten
      }

      foreach aktHand {hand hand2} {
        if {[info exists fuelle($aktHand)] && $fuelle($aktHand) != " "} {
          stelleKarteDar $flaeche neu \
                      [expr {$fuelle($aktHand) ? "ruecken" : "h"}] $aktHand \
                      $pos(x,$aktHand) $pos(y,$aktHand)
        }
      }
      maleLeereStoecke
      maleKartenAnz
      for {set aktSatz 0} {$aktSatz < $kartenSatz($patArt)} {incr aktSatz} {
        foreach aktFarbe {c h p k} {
          set aktStock $aktSatz,$aktFarbe
          for {set aktY 0} {$aktY < $fuelle($aktStock)} {incr aktY} {
            set karte    $stapel($aktStock,$aktY)
            maleKarte    $karte neu $aktStock
            steckInStock $aktStock $karte 0 0
          }
        }
      }
      update
      for {set aktArb 0} {$aktArb < $exist($patArt,arbeit)} {incr aktArb} {
        set aktStapel arbeit$aktArb
        if {$fuelle($aktStapel)} {
          set karte    $stapel($aktStapel,0)
          maleKarte    $karte neu $aktStapel
          steckInStock $aktStapel $karte 0 0
        }
      }
      foreach sonderStapel {talon talon2 strafe strafe2} {
        if {[info exists fuelle($sonderStapel)]} {
          set verdeckt($sonderStapel) 0
          for {set aktI 0} {$aktI < $fuelle($sonderStapel)} {incr aktI} {
            maleKarte $stapel($sonderStapel,$aktI) neu $sonderStapel
          }
        }
      }
      update
      for {set aktStapel 0} {$aktStapel < $stapelAnz($patArt)} {incr aktStapel} {
        for {set aktY 0} {$aktY < $fuelle($aktStapel)} {incr aktY} {
          set karte $stapel($aktStapel,$aktY)

          if {$aktY < $verdeckt($aktStapel)} {
            stelleKarteDar $flaeche neu ruecken $karte \
                      [stapelX $aktStapel] [stapelY $aktStapel $aktY]
            nichtVerschiebbar $karte
          } else {
            maleKarte $karte neu $aktStapel $aktY
          }
        }
        pruefFreieKarte $aktStapel minusEins
        update idletask
      }
      if {$streitArt($patArt)} {
        maleZugbestaetigungsknopf
      }
    }
    proc maleKartenAnz {} {
      global flaeche pos stapelBreite streitArt patArt fuelle

      $flaeche create text [expr {$pos(x,hand)+$stapelBreite/2}] \
                           [expr {$pos(y,hand)-10}]              \
                           -text $fuelle(hand) -tags kartenAnz
      if {$streitArt($patArt)} {
        $flaeche create text [expr $pos(x,hand2)+$stapelBreite/2] \
                             [expr {$pos(y,hand2)-10}]            \
                             -text $fuelle(hand2) -tags kartenAnz2
      }
    }
    proc maleKarte {karte neu aktStapel {aktY 0}} {
      global flaeche fuelle

      if {![string compare $neu alt]} {
        $flaeche coords $karte [stapelX $aktStapel] [stapelY $aktStapel $aktY]
        $flaeche raise  $karte
      } else {
        stelleKarteDar $flaeche $neu [string range $karte 0 1] $karte \
                    [stapelX $aktStapel] [stapelY $aktStapel $aktY]
      }
      verschiebbar $karte $aktStapel $aktY
    }
    proc maleRuecken {karte aktStapel {aktY -1}} {
      global flaeche

      stelleKarteDar $flaeche  [expr {$aktY >= 0 ? "neu" : "anders"}] ruecken \
             $karte [stapelX $aktStapel] [stapelY $aktStapel $aktY]
      nichtVerschiebbar $karte
    }
    proc pfeil {vonStap vonY nachStap nachY festeZeit} {
      global warteZeit flaeche kartenHoehe stapelBreite demoAktiv

      set tausendstel [expr $warteZeit*100]
      if {$festeZeit && $tausendstel <= 0} {set tausendstel 0}

      set xOffset [expr {$stapelBreite/2}]
      set yOffset [expr {$kartenHoehe/9}]
      set vonX    [expr {[stapelX $vonStap]        +$xOffset}]
      set vonY    [expr {[stapelY $vonStap  $vonY] +$yOffset}]
      set nachX   [expr {[stapelX $nachStap]       +$xOffset}]
      set nachY   [expr {[stapelY $nachStap $nachY]+$yOffset}]

      if {$tausendstel || !$festeZeit} {
        $flaeche create line $vonX $vonY $nachX $nachY -tag pfeil -width 5 \
            -arrow last -arrowshape {10 10 5}
      }
      update idletasks
      if {$festeZeit} {
        # Ab tk3.3 reagiert after nicht mehr auf Events. Schade eigentlich...
        if {!$demoAktiv} return
        after $tausendstel set warteZeitVorbei 1
        tkwait variable warteZeitVorbei
      } else {
        aktiviereBindings "+set pfeilAus 0"
        tkwait variable pfeilAus
        aktiviereBindings
      }
      if {$tausendstel || !$festeZeit} {
        $flaeche delete pfeil
      }
    }
    proc mischComic {x y hand richtung} {
      global stapelBreite mitMischComic flaeche

      stelleKarteDar $flaeche neu ruecken $hand $x $y
      if {!$mitMischComic} return

      stelleKarteDar $flaeche neu ruecken beweg $x $y

      # Falls die Hand links unten ist, muss nach rechts gemischt werden...
      set hin     [expr {$richtung == "rechts" ? 3 : -3}]
      set zurueck [expr {-$hin}]

      for {set j 0} {$j < 8} {incr j} {
        $flaeche raise beweg
        for {set i 0} {$i < 20} {incr i} {
          $flaeche move beweg $hin 0
          update idletask
        }
        $flaeche lower beweg
        for {set i 0} {$i < 20} {incr i} {
          $flaeche move beweg $zurueck 0
          update idletask
        }
      }
      $flaeche delete beweg
    }
    proc verteilComic {vonStapel vonY nachStapel nachY {kartenAnz 0} args} {
      global stapelBreite kartenHoehe flaeche mitBewegComic stapel \
             bildBreite bildHoehe

      if {!$mitBewegComic} return

      if {[llength $args] >= 2} {
        spielXY x y [lindex $args 0] [lindex $args 1]
      } else {
        set x [stapelX $vonStapel]
        set y [stapelY $vonStapel $vonY]
      }
      # Das X- und Y-Inkrement zwischen Start und Ziel berechnen.
      set xDist [expr {[stapelX $nachStapel]-$x}]
      set dx    [expr {$bildBreite/20}]
      if {$xDist < 0}   {set dx -$dx}
      if {abs($xDist) < abs($dx)} return
      set yDist [expr {[stapelY $nachStapel $nachY]-$y}]
      set dy    [expr {$yDist*$dx/$xDist}]

      if {$kartenAnz} {
        set tag stap
        for {set aktY $vonY} {$aktY < $vonY+$kartenAnz} {incr aktY} {
          set karte $stapel($vonStapel,$aktY)
          $flaeche raise $karte
          $flaeche itemconfigure $karte -tags "$karte $tag"
        }
      } else {
        set tag beweg
        stelleKarteDar $flaeche neu ruecken $tag $x $y
      }

      for {set sx $dx} {abs($sx) < abs($xDist-$dx/2)} {incr sx $dx} {
        $flaeche move $tag $dx $dy
        update idletask
      }

      if {$kartenAnz} {
        $flaeche dtag $tag $tag
      } else {
        $flaeche delete $tag
      }
    }
    proc schatten {} {
        global schatten text

        set w .schatten

        catch {destroy $w}
        toplevel $w
        wm title $w    $text(schatten)
        wm iconname $w $text(schatten)

        button $w.ok -text $text(okay) -command "destroy $w"
        pack   $w.ok -fill x -side bottom

        message $w.msg -font -Adobe-times-medium-r-normal--*-180* -width 4i \
                  -text $text(schatten)
        pack  $w.msg -fill x

        frame $w.f1 -bd 1
        pack  $w.f1 -expand 1 -fill x

        label $w.f1.label -text $text(schattenGroesse)
        pack  $w.f1.label

        scale $w.f1.scale -orient horizontal -length 280 -from 0 -to 20 \
                -command "setzSchatten" -tickinterval 10
        $w.f1.scale set $schatten
        pack $w.f1.scale

        wm withdraw $w
        update idletasks
        set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
                  - [winfo vrootx [winfo parent $w]]]
        set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
                  - [winfo vrooty [winfo parent $w]]]
        wm geom $w +$x+$y
        wm deiconify $w
      }
      proc setzSchatten {wert} {
        global schatten text

        set schatten $wert
        confMenueEntry .menu.karten.m "$text(schatten:)*" \
                       -label "$text(schatten:) $schatten..."
      }
      proc warteZeit {} {
        global warteZeit text

        set w .warte
        catch {destroy $w}
        toplevel $w
        wm title $w    $text(demonstration)
        wm iconname $w $text(demonstration)

        button $w.ok -text $text(okay) -command "destroy $w"
        pack   $w.ok -fill x -side bottom

        message $w.msg -font -Adobe-times-medium-r-normal--*-180* -width 4i \
                  -text $text(demonstration)
        pack  $w.msg -fill x

        frame $w.f1 -bd 1
        pack  $w.f1 -expand 1 -fill x

        label $w.f1.label -text $text(zeitverzoegerung)
        pack  $w.f1.label

        scale $w.f1.scale -orient horizontal -length 280 -from 0 -to 100 \
                -command "setWarteZeit" -tickinterval 20
        $w.f1.scale set $warteZeit
        pack $w.f1.scale

        wm withdraw $w
        update idletasks
        set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
                  - [winfo vrootx [winfo parent $w]]]
        set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
                  - [winfo vrooty [winfo parent $w]]]
        wm geom $w +$x+$y
        wm deiconify $w
      }
      proc setWarteZeit {wert} {
        global warteZeit text

        set warteZeit $wert
        confMenueEntry .menu.pat.m $text(warte)*$text(sekunden)... \
                       -label $text(warte)[expr {double($wert)/10}]$text(sekunden)...
      }
    proc demo {} {
      global fuelle verdeckt stapel stapelAnz streitArt patArt demoAktiv \
             exist kommandoNachDemo

      initVorschlaege
      set dieseRundeLebt 1
      set demoAktiv      1
      menueHilfe [expr {$streitArt($patArt) ? "streit" : "demo"}]
      aktiviereBindings "set demoAktiv $streitArt($patArt)"
      aktiviereGeben hand ein

      # Nun beschaeftigen wir uns solange alleine, bis die Benutzerin andeutet,
      # dass ihr die Demo langweilig ist, oder wir in einer Streitpatience
      # nicht mehr weiter duerfen.
      while {$demoAktiv} {
        set gezogen [vorschlag 1]

        # Eine Denkpause ist unumgaenglich, damit die Spielerin mit ihren
        # Tastendruecken ueberhaupt durchkommt.
        update; kurzePause; if {!$demoAktiv} break

        set gewonnen [expr {$streitArt($patArt) && [checkSpielEnde 1]}]
        if {$gewonnen} {
          set gezogen 0
          aktiviereGeben hand aus
        }

        if {$gezogen} {
          set dieseRundeLebt 1
          for {set aktStap 0} {$aktStap < $stapelAnz($patArt)} {incr aktStap} {
            set oben [expr $fuelle($aktStap)-1]
            if {$oben < 0} continue
            if {$oben < $verdeckt($aktStap)} {
              drehUm $stapel($aktStap,$oben) $aktStap 1
              initVorschlaege
              update
            }
          }
        } elseif {$streitArt($patArt)} {
          if {![istGebenAktiv]} {
            set demoAktiv 0
            schaltSchachuhrAufMensch 0
          }
        } else {
          initVorschlaege
          if {$fuelle(hand) > 0} {
            if {[geben hand]} {set dieseRundeLebt 1}
          } elseif {$exist($patArt,talon) && $fuelle(talon) > 0 && $dieseRundeLebt} {
            set dieseRundeLebt [handNachTalon hand talon 1]
          } else {
            starteSpielInDemo
          }
        }
      }
      catch {global flaeche; $flaeche delete pfeil}
      menueHilfe aus
      aktiviereBindings
      
      if {[info exists kommandoNachDemo]} {
        set aktKomm $kommandoNachDemo
        unset kommandoNachDemo
        eval $aktKomm
      }
    }
    proc starteSpielInDemo {} {
      global harterTest demoAktiv

      if {[info exists harterTest]} {
        schrZustand "t_ende.pat" 0
        zumAnfang
      }

      if {$demoAktiv} {
        starteSpiel 0
      
        if {[info exists harterTest]} {
          schrZustand "t_start.pat" 0
        }
      }
    }
    proc streitbar {{vorgabe keine}} {
        global streitbar ich rpcPeer rpcPort meinRechner text

        if {[string compare $vorgabe keine]} {
          if {[info tclversion] < 7.5 && $vorgabe} {
            nurGegenComputer
            set streitbar 0
            return
          }
          set streitbar $vorgabe
        }

        if {$streitbar} {
          global patiencePath

          set daemonPort [option get .patience daemonPort DaemonPort]
          if {[catch {set patDaemon [dp_MakeRPCClient localhost $daemonPort]}]} {
            set daemon $patiencePath/patd
            if {![file executable $daemon]} {
              keinDaemon "$daemon: not executable"
              set streitbar 0
            } else {
              set kommando "$daemon $daemonPort 2>&1 &"
              if {[catch {open "| /bin/sh -c \"$kommando 2>&1\""} prot]} {
                keinDaemon "patience: $prot"
                set streitbar 0
                catch {close $prot}
              } else {
                set zeile [gets $prot]
                if {![string match *successfully* $zeile]} {
                  keinDaemon $zeile
                  set streitbar 0
                  close $prot
                }
              }
            }
          } else {
            dp_CloseRPC $patDaemon
          }
          if {![info exists rpcPeer]} {
            set socketPortList [dp_MakeRPCServer 0 dp_CheckHost none 1]
            set rpcPeer [lindex $socketPortList 0]
            set rpcPort [lindex $socketPortList 1]
          }
        }
        if {$vorgabe != 0} {
          set daemonPort [option get .patience daemonPort DaemonPort]
          if {[catch [list dp_MakeRPCClient localhost $daemonPort] patDaemon]} {
            keinDaemon $patDaemon
            set streitbar 0
          } else {
            set meldeProc  [expr {$streitbar ? "patAnmelden" : "patAbmelden"}]
            dp_RPC $patDaemon $meldeProc $ich $rpcPort
            dp_CloseRPC $patDaemon
          }
        }
        if {[catch {info hostname} meinRechner]} {
          global tcl_platform

          set meinRechner localhost
          switch $tcl_platform(platform) {
            unix {catch {set meinRechner [exec hostname]}}
          }
        }
      }
    proc senden {args} {
      global text kontakt

      if {![info exists kontakt(unserProt)]} {return 1}

      set fhl [catch {eval dp_RPC $kontakt(deinPeer) after 0 $args} meldung]

      if {$fhl} {
        set kontakt(text) $text(keineReaktion)
        kontaktDialog sendeFehler 0 2 \
            [list $text(neuSenden)     "if {!\[senden $args]} kontaktDialogEnde"] \
            [list $text(neuAnfragen)   "kontaktAnfragen"] \
            [list $text(gegenComputer) "kontaktAbbauen 1"]
      }
      return $fhl
    }
    proc kontakt {} {
      global text kontakt gegner

      if {![info exists gegner(port)]} {
        set gegner(port) [option get .patience daemonPort DaemonPort]
      }

      set kontakt(text) ""
      kontaktDialog kontakt 1 1 \
            [list $text(abbruch)  "kontaktAbbruch willDochNicht 0"] \
            [list $text(anfragen) "kontaktAnfragen"]
    }
    proc ping {} {
      senden pid
    }
    proc abgleich {} {
      global text kontakt

      set kontakt(text) $text(gegnerFragen)
      kontaktDialog abgleich 0 1 \
        [list $text(abbruch) "kontaktDialogEnde; senden kontaktDialogEnde"]
      senden abgleichOkay
    }
    proc abgleichOkay {} {
      global text kontakt

      set kontakt(text) $text(zustandUebernehmen)
      kontaktDialog abgleich 0 1 \
            [list $text(nein) "keinAbgleich"  ] \
            [list $text(ja)   "schickAbgleich"]
    }
    proc keinAbgleich {} {
      kontaktDialogEnde
      senden set kontakt(text) \$text(nichtEinverstanden)
    }
    proc schickAbgleich {} {
      kontaktDialogEnde
      senden set kontakt(text) \$text(ja)
      senden abgleichen
    }
    proc abgleichen {} {
      lappend kommando "initZustand"
      schrZustand kommando 1
      lappend kommando "bildNeu; deaktiviereGegnerStapel"
      kontaktDialogEnde
      senden [join $kommando "\n"]
    }
    proc kontaktBeenden {neuesSpiel} {
      global text kontaktBeendenVar ich kontakt meinRechner

      set kontaktBeendenVar 0

      if {$neuesSpiel} {
        set kontakt(text) $text(kontaktNeustart)
        kontaktDialog aufgabe 1 2 \
            [list $text(abbruch)    "kontaktDialogEnde"] \
            [list $text(neuesSpiel) "kontaktNeustart"] \
            [list $text(beenden)    "kontaktAbbruch hatAufgegeben 1"]
      } else {
        set kontakt(text) $text(kontaktAbbrechen)
        kontaktDialog aufgabe 1 1 \
            [list $text(abbruch) "kontaktDialogEnde"] \
            [list $text(beenden) "kontaktAbbruch hatAufgegeben 1"]
      }
      tkwait window .kontakt
      return $kontaktBeendenVar
    }
    proc kontaktNeustart {} {
      global kontaktBeendenVar kontakt text

      set kontakt(text) $text(gegnerAnrufen)

      senden kontaktNeuesSpiel
    }
    proc kontaktNeuesSpiel {} {
      global kontakt text

      set kontakt(text) $text(neuerAnfang)

      set a "kontaktDialogEnde; senden set kontakt(text) \\\$text("
      set z ")"
      kontaktDialog kontakt 0 1 \
            [list $text(nein)       "${a}nichtEinverstanden${z}"] \
            [list $text(neuesSpiel) "${a}gehtLos${z}; starteSpiel 0"]
    }
    proc kontaktAnfragen {} {
      global gegner text ich patArt \
             streitbar kontakt rpcPort rpcPeer meinRechner

      if {![string length $gegner(name)] || ![string length $gegner(host)]
          || ![string length $gegner(port)]} {
        set kontakt(text) $text(gegnerAngeben)
        return
      }
      if {!$streitbar} {
        streitbar 1
      }
      if {![string compare $gegner(host) localhost]} {
        set gegner(host) $meinRechner
      }
      if {![string compare [string tolower $ich] \
                           [string tolower $gegner(name)]]} {
        set kontakt(text) $text(gegenSelbst)
        return
      }    

      catch {unset kontakt(unserProt)}

      if {[catch {set patdPeer [dp_MakeRPCClient $gegner(host) $gegner(port)]}]
          || $patdPeer == 0} {
        set kontakt(text) $text(keinDaemonDa)
        return
      }
      if {[catch {set kontakt(deinPort) \
                    [dp_RPC $patdPeer patPortVon $gegner(name)]}]
          || $kontakt(deinPort) == 0} {
        set kontakt(text) $text(gegnerUnbekannt)
        return
      }
      dp_CloseRPC $patdPeer

      if {$rpcPort == 0} {
        set kontakt(text) $text(meinPortFehlt)
        return
      }
      set kontakt(unserProt) dp_RPC
      set kontakt(meinPort)  $rpcPort
      set kontakt(meinPeer)  $rpcPeer
      if {[catch {set kontakt(deinPeer) \
                        [dp_MakeRPCClient $gegner(host) $kontakt(deinPort)]}]
          || $kontakt(deinPeer) == 0} {
        set kontakt(text) $text(deinPeerFehlt)
        return
      }

      if {[info exists kontakt(unserProt)]} {
          set kontakt(text) $text(gegnerAnrufen)
          update
          senden kontaktGefaellig $ich $meinRechner $patArt $kontakt(meinPort)
      } else {
        set kontakt(text) $text(gegnerNichtDa)    
      }
    }
    proc kontaktAbbruch {sendeText weiter} {
      global kontakt kontaktBeendenVar ich du meinRechner

      set kontaktBeendenVar 1

      if {[info exists kontakt(unserProt)]} {
        senden undTschuess $sendeText $ich@$meinRechner
        kontaktAbbauen $weiter
      } else {
        kontaktDialogEnde
      }
    }
    proc kontaktAbbauen {weiter} {
      global text ich du aktiverSpieler streitArt patArt kontakt

      if {[info exists kontakt(unserProt)]} {
        # baue RPC-Verbindung ab
        catch {close $kontakt(deinPeer)}
      }
      catch {unset kontakt}

      kontaktDialogEnde
      if {!$weiter || !$streitArt($patArt)} return

      set du $text(computer)
      aktiviereMenuepunkteFuerStreit
      if {[string compare $aktiverSpieler $ich]} {
        set aktiverSpieler $du
        demo
      }
    }
    proc undTschuess {sendeText anfrager} {
      global kontakt text du deinRechner

      set kontakt(text) $text($sendeText)
      set weiter [expr {[info exists du] && [info exists kontakt(deinRechner)]
                        && ![string compare $anfrager $du@$kontakt(deinRechner)]}]
      kontaktDialog aufgabe 0 0 \
            [list $text(weiterOhneIhn) "kontaktAbbauen $weiter"]
      catch {unset kontakt(unserProt)}
    }
    proc bistDuEinePatience {} {
      global text ich streitbar

      if {$streitbar} {
        return jawoll,user=$ich
      } else {
        return jawoll,user=$ich,habAberKeineLust
      }
    }
    proc kontaktGefaellig {anfrager host streitPat args} {
      global text gegner patName sprache kontakt

      if {[catch {set port [dp_MakeRPCClient $host $args]} fhl]} {
        set kontakt(text) $fhl; return
      }
      if {$port == 0} {
        set kontakt(text) "Oops, no port"; return
      }

      if {[info exist kontakt(unserProt)]} {
        set fhl besetzt
      } elseif {![info exists patName($streitPat,$sprache)]} {
        set fhl unbekPatArt
      } else {
        set fhl iO
      }

      if {[string compare $fhl iO]} {
        catch {$prot $port eval set kontakt(text) \$text($fhl)} msg
        if {![string compare $prot dp_RPC]} {
          close $port
        }
        return
      }

      set kontakt(unserProt) dp_RPC
      set kontakt(deinPeer)  $port

      set a "senden set kontakt(text) \\\$text("
      set z "); unset kontakt(unserProt) kontakt(deinPeer); kontaktDialogEnde"
      set kontakt(text) $text(erMagSpielen)
      set gegner(name) $anfrager
      set gegner(host) $host

      kontaktDialog spielGefaellig 0 2 \
        [list $text(nein)    "${a}hatKeineLust${z}"] \
        [list $text(spaeter) "${a}spaeter${z}"]      \
        [list $text(okay)    "kontaktDialogEnde; \
                              kontaktJa $anfrager $host $streitPat"]
    }
    proc kontaktJa {name host streitPat} {
      global du kontakt demoAktiv kommandoNachDemo

      if {$demoAktiv} {
        set demoAktiv        0
        set kommandoNachDemo "kontaktJa $name $host $streitPat"
        return
      }

      set du                   $name
      set kontakt(deinRechner) $host

      senden set kontakt(text) \$text(gehtLos)

      starteSpiel 0 $streitPat
    }
    proc kontaktDialog {titel mitEingabe default args} {
      global gegner text tk_version

      set w .kontakt
      set dialogWarDa [winfo exists $w]
      if {$dialogWarDa} {
        if {![string compare [wm state $w] iconic]} {wm deiconify $w}
        raise $w
        catch {foreach but [pack slaves $w.default] {destroy $but}}
        catch {foreach but [pack slaves $w.buttons] {destroy $but}}
      } else {
        toplevel    $w
        wm title    $w $text($titel)
        wm iconname $w $text($titel)

        frame $w.eingabe -relief raised -bd 1
        label $w.ausgabe -relief raised -bd 1 -textvariable kontakt(text)
        frame $w.buttons -relief raised -bd 1
        pack  $w.eingabe $w.ausgabe $w.buttons \
                    -fill x -expand 1 -ipadx 5m -ipady 2m

        pack [frame $w.eingabe.user] -fill x -expand 1
        pack [frame $w.eingabe.host] -fill x -expand 1
        pack [frame $w.eingabe.port] -fill x -expand 1

        label $w.eingabe.user.text -text $text(gegnerName)
        entry $w.eingabe.user.feld -textvariable gegner(name) -width 40
        pack  $w.eingabe.user.feld $w.eingabe.user.text -side right -padx 10

        label $w.eingabe.host.text -text $text(gegnerHost)
        entry $w.eingabe.host.feld -textvariable gegner(host) -width 40
        pack  $w.eingabe.host.feld $w.eingabe.host.text -side right -padx 10

        label $w.eingabe.port.text -text $text(gegnerPort)
        entry $w.eingabe.port.feld -textvariable gegner(port) -width 40
        pack  $w.eingabe.port.feld $w.eingabe.port.text -side right -padx 10
      }
      foreach name {user host port} {
        $w.eingabe.$name.feld configure \
            -state [expr {$mitEingabe ? "normal" : "disabled"}]
      }

      set i 0
      foreach but $args {
        button $w.button$i -text [lindex $but 0] -command [lindex $but 1]
        if {$i == $default} {
          frame $w.default -relief sunken -bd 1
          raise $w.button$i $w.default
          pack $w.default -in $w.buttons -side left -expand 1 -padx 3m -pady 2m
          pack $w.button$i -in $w.default -padx 2m -pady 2m
          foreach name {user host port} {
            bind $w.eingabe.$name.feld <Return> \
                    "$w.button$i flash; [lindex $but 1]"
          }
        } else {
          pack $w.button$i -in $w.buttons -side left -expand 1 -padx 3m -pady 2m
        }
        incr i
      }

      if {!$dialogWarDa} {
        bind $w <Any-Enter> "focus $w.eingabe.user.feld"
        wm withdraw $w
        update idletasks
        set x [expr [winfo screenwidth $w]/2 - [winfo reqwidth $w]/2 \
                  - [winfo vrootx [winfo parent $w]]]
        set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
                  - [winfo vrooty [winfo parent $w]]]
        wm geom $w +$x+$y
        wm deiconify $w
      }
    }
    proc kontaktDialogEnde {} {
      catch {destroy .kontakt}
    }
    proc werFaengtAn {} {
      global ich du kartenHoehe stapelBreite flaeche pos stapel \
             aktiverSpieler ich du text

      set aktiverSpieler $text(werFaengtAn)
      maleZugbestaetigungsknopf
      $flaeche create text [expr {$pos(x,hand) +0.5*$stapelBreite}] \
                           [expr {$pos(y,hand) -10}] -text $du  -tag du
      $flaeche create text [expr {$pos(x,hand2)+0.5*$stapelBreite}] \
                           [expr {$pos(y,hand2)-11}] -text $ich -tag ich
      update

      mischComic $pos(x,hand)  $pos(y,hand)  hand  links
      mischComic $pos(x,hand2) $pos(y,hand2) hand2 rechts 

      verteilComic hand  0 talon  0
      stelleKarteDar $flaeche neu [string range $stapel(deinStart) 0 1] \
                     talon  $pos(x,talon)  $pos(y,talon)
      verteilComic hand2 0 talon2 0
      stelleKarteDar $flaeche neu [string range $stapel(meinStart) 0 1] \
                     talon2 $pos(x,talon2) $pos(y,talon2)

      set meinStartWert [expr {[kartenWert $stapel(meinStart)]*13
                             + [farbRang   $stapel(meinStart)]}]
      set deinStartWert [expr {[kartenWert $stapel(deinStart)]*13
                             + [farbRang   $stapel(deinStart)]}]
      set aktiverSpieler [expr {$meinStartWert > $deinStartWert ? $ich : $du}]

      after 3000 {set kurzePauseZumDruecken 1}
      tkwait variable kurzePauseZumDruecken

      $flaeche delete talon talon2 hand hand2 ich du
    }
    proc informiereStreitGegner {zugArt args} {
      switch $zugArt {
        Karte {
          set karte     [lindex $args 0]
          set altStapel [lindex $args 1]
          switch $altStapel {
            hand2   {set altStapel hand}
            talon2  {set altStapel talon}
            strafe2 {set altStapel strafe}
          }
          set neuStapel [lindex $args 2]
          switch $neuStapel {
            hand    {set neuStapel hand2}
            talon   {set neuStapel talon2}
            strafe  {set neuStapel strafe2}
          }
          set kommando "zurueckKarte $karte $neuStapel $altStapel 1"
        }
        Stapel {
          global fuelle

          set altStapel [lindex $args 0]
          set neuStapel [lindex $args 2]
          set altY      $fuelle($altStapel)
          set kommando "zurueckStapel $neuStapel $altY $altStapel"
        }
        Geben {
          set meineHand [lindex $args 0]
          set deineHand [expr {$meineHand == "hand2" ? "hand" : "hand2"}]
          set kommando "geben $deineHand"
        }
        HandNachTalon {
          set meineHand [lindex $args 0]
          set deineHand [expr {$meineHand == "hand2"  ? "hand"  : "hand2"}]
          set meinTalon [lindex $args 1]
          set deinTalon [expr {$meinTalon == "talon2" ? "talon" : "talon2"}]
          set kommando "handNachTalon $deineHand $deinTalon 0"
        }
        DrehUm {
          set kommando "drehUm $args 0"
        }
        SchachUhr {
          set kommando "schaltSchachuhrAufMensch 1"
        }
        Beileid {
          set kommando "streitBeileid $args"
        }
      }
      update
      senden $kommando
    }
    proc vorschlag {mitSchieben} {
        global flaeche vorschlagZaehler vorschlaege zugNr \
               vorschlaegeAnzeigen vorschlagsReihe patArt auchBild

        if {[info exists vorschlagsReihe($zugNr,priori)]} {
          set vorschlaege(zugNr)  $zugNr
          set vorschlaege(anzahl) 1
          set vorschlaege(index)  0
          foreach elem {priori prioriExpr pfeil schieb} {
            set vorschlaege(0,$elem) $vorschlagsReihe($zugNr,$elem)
          }
        } elseif {[info exists vorschlaege] && $vorschlaege(zugNr) == $zugNr
                  && $vorschlaege(anzahl) > 0} {
          if {$mitSchieben || [incr vorschlaege(index)] >= $vorschlaege(anzahl)} {
            # Wir sind einmal rum, und fangen wieder vorne an.
            set vorschlaege(index) 0      
          }
          catch {global flaeche; $flaeche delete pfeil}
        } else {
          berechneVorschlaege $mitSchieben
        }
        if {[info exists vorschlaegeAnzeigen]} {
          set result "Vorschlaege($vorschlaege(zugNr)):\n"
          for {set aktV 0} {$aktV < $vorschlaege(anzahl)} {incr aktV} {
            append result [format "%3d: (%3d) %s <%s>\n" $aktV       \
                $vorschlaege($aktV,priori) $vorschlaege($aktV,pfeil) \
                $vorschlaege($aktV,prioriExpr)]
          }

          if {![winfo exist .vorschlaege]} {
            toplevel .vorschlaege
            text .vorschlaege.text -relief raised -bd 2 -width 40 -height 10 \
                -font -Adobe-times-medium-r-normal--*-140*
            pack .vorschlaege.text
          } else {
            .vorschlaege.text configure -state normal
            .vorschlaege.text delete 1.0 end
          }
          .vorschlaege.text insert 1.0 $result
          .vorschlaege.text configure -state disabled
        }
        incr vorschlagZaehler
        if {$vorschlaege(anzahl) == 0} {
          if {!$mitSchieben} {
            global tk_version
            if {$tk_version >= 4.0} {
              bell
            } else {
              puts -nonewline "\a"
            }
          }
          return 0
        } else {
          global vorschlagFertig pfeilFertig aktPfeil

          if {$mitSchieben} {
            set aktSchieb $vorschlaege($vorschlaege(index),schieb)
            set aktPfeil  $vorschlaege($vorschlaege(index),pfeil)

            if {![info exist vorschlagsReihe([expr {$zugNr+1}],priori)]} {
              set auchBild 0

              eval $aktSchieb
              set vorschlagFertig 0
              after 1 {
                berechneVorschlaege 1
                set vorschlagFertig 1
              }
              eval $aktPfeil 1
              if {!$vorschlagFertig} {
                tkwait variable vorschlagFertig
              }
              while {[info exists vorschlaegeAnzeigen] && $vorschlaegeAnzeigen > 1} {
                tkwait variable vorschlaegeAnzeigen
              }
              zurueckZumZug [expr {$zugNr-1}]
              set auchBild 1
            } else {
              eval $aktPfeil 1
            }
            eval $aktSchieb
          } else {
            eval $vorschlaege($vorschlaege(index),pfeil) 0
          }
          return 1
        }
      }
    proc berechneVorschlaege {mitSchieben} {
      global flaeche vorschlagZaehler vorschlaege zugNr streitArt \
             vorschlaegeAnzeigen vorschlagsReihe stapelAnz patArt \
             stapel fuelle verdeckt teilStapelUnterst exist       \
             prioritaet teilStapelReihe leerStapelFeld            \
             talonAbraeumbar auchBild teilY stockReihe

      initVorschlaege $zugNr
        set sonderStapelListe ""
          if {$exist($patArt,strafe)} {lappend sonderStapelListe "strafe  strafe"}
          if {$exist($patArt,talon)}  {lappend sonderStapelListe "talon   talon"}
          for {set aktArb 0} {$aktArb < $exist($patArt,arbeit)} {incr aktArb} {
            lappend sonderStapelListe "arbeit$aktArb arbeit"
          }
        if {![string match *,kompl $stockReihe($patArt)]} {
          foreach stapelBeschreibung $sonderStapelListe {
            set sonderStapel [lindex $stapelBeschreibung 0]
            if {$fuelle($sonderStapel) > $verdeckt($sonderStapel)} {
              set oben [expr $fuelle($sonderStapel)-1]
              set karte $stapel($sonderStapel,$oben)
              set neuStock [kannAufStock $karte]
              if {[string length $neuStock]} {
                set stapelKlasse [lindex $stapelBeschreibung 1]
                vorschlagAufStock $sonderStapel $stapelKlasse $oben $karte \
                                  $neuStock [stockBonus $karte]
              }
            }
          }
        }
        set leerStapelFeld {}
        if {[string compare $teilStapelReihe($patArt) garNicht]} {
          set maxTeilStapelAnz 13
          for {set aktStapel 0} {$aktStapel<$stapelAnz($patArt)} {incr aktStapel} {
            set teilY($aktStapel) $teilStapelUnterst($aktStapel)
          }
        } else {
          set maxTeilStapelAnz 1
          for {set suchStap 0} {$suchStap < $stapelAnz($patArt)} {incr suchStap} {
            if {$fuelle($suchStap) == 0} {
              lappend leerStapelFeld $suchStap
              incr maxTeilStapelAnz $maxTeilStapelAnz
            }
          }
          for {set aktStapel 0} {$aktStapel<$stapelAnz($patArt)} {incr aktStapel} {
            set aktY [expr {$fuelle($aktStapel)-1}]
            while {$aktY > $verdeckt($aktStapel)
                   && [passt $stapel($aktStapel,$aktY) \
                             $stapel($aktStapel,[expr {$aktY-1}]) $aktStapel]} {
              incr aktY -1
            }
            set teilY($aktStapel) $aktY
          }
        }
        for {set aktStapel 0} {$aktStapel<$stapelAnz($patArt)} {incr aktStapel} {
          set oben [expr $fuelle($aktStapel)-1]
          if {[string match *,kompl $stockReihe($patArt)]} {
            set oben [expr {$oben-13}]
          }
          if {$oben >= 0 && $oben >= $verdeckt($aktStapel)
              && $oben >= $teilStapelUnterst($aktStapel)} {
            # Es liegt mindestens eine Karte auf dem Stapel.
            set aktKarte $stapel($aktStapel,$oben)
            set neuStock [kannAufStock $aktKarte]
            if {[string length $neuStock]} {
              vorschlagAufStock $aktStapel stapel $oben $aktKarte \
                                $neuStock [stockBonus $aktKarte]
            }
            switch -glob $aktKarte {
              ?c? {set andereFarbe h}
              ?h? {set andereFarbe c}
              ?p? {set andereFarbe k}
              ?k? {set andereFarbe p}
            }
            set andereKarte [string index $aktKarte 0]$andereFarbe
            set neuStock [kannAufStock ${andereKarte}0]
            if {[string length $neuStock]} {
              for {set neuStapel 0} {$neuStapel<$stapelAnz($patArt)} {incr neuStapel} {
                if {![string compare $neuStapel $aktStapel]} continue
                for {set aktY [expr $fuelle($neuStapel)-2]} \
                    {$aktY >= $fuelle($neuStapel)-$maxTeilStapelAnz-1 &&
                     $aktY >= $teilY($neuStapel)} {incr aktY -1} {
                  set verglKarte [string range $stapel($neuStapel,$aktY) 0 1]
                  if {![string compare $andereKarte $verglKarte]} {
                    vorschlagSchieb [expr {$fuelle($neuStapel)-($aktY+1)}]    \
                                    $neuStapel stapel [expr {$aktY+1}]    \
                                    $aktStapel stapel $fuelle($aktStapel) \
                                    $prioritaet(andereFarbePasst)+[stockBonus $andereKarte]
                  }
                }
              }
            }
          }
        }
        if {!$talonAbraeumbar($patArt) && [istGebenAktiv]} {
          vorschlagSchieb 1 hand  talon [expr {$fuelle(hand)-1}] \
                            talon talon $fuelle(talon) 1000
        }
        if {!$mitSchieben || $vorschlaege(anzahl) == 0} {
          if {$streitArt($patArt)} {
            foreach ziel {strafe talon} {
              if {!$exist($patArt,$ziel)} continue
              set zielStapel ${ziel}2
              set zielOben [expr {$fuelle($zielStapel) - 1}]
              if {$zielOben >= 0} {
                set zielKarte $stapel($zielStapel,$zielOben)
                for {set aktStapel 0} {$aktStapel<$stapelAnz($patArt)} {incr aktStapel} {
                  set oben [expr {$fuelle($aktStapel)-1}]
                  if {$oben >= 0 && $oben >= $verdeckt($aktStapel)
                      && [passt $stapel($aktStapel,$oben) $zielKarte $zielStapel]} {
                    vorschlagSchieb 1 $aktStapel stapel $oben \
                                      $zielStapel $ziel $fuelle($zielStapel) \
                                      [stapelBonus $aktStapel $oben]
                  }
                }
                foreach aktStapel {strafe talon} {
                  if {!$exist($patArt,$aktStapel)} continue
                  set oben [expr {$fuelle($aktStapel)-1}]
                  if {$oben >= 0
                      && [passt $stapel($aktStapel,$oben) $zielKarte $zielStapel]} {
                    vorschlagSchieb 1 $aktStapel $aktStapel $oben \
                                      $zielStapel $ziel $fuelle($zielStapel) \
                                      [stapelBonus $aktStapel $oben]
                  }
                }
              }
            }
          }
          foreach stapelBeschreibung $sonderStapelListe {
            set sonderStapel [lindex $stapelBeschreibung 0]
            if {$fuelle($sonderStapel) > $verdeckt($sonderStapel)} {
              set oben [expr $fuelle($sonderStapel)-1]
              set karte $stapel($sonderStapel,$oben)
              set neuStapel 0
              while {42} {
                wohinBewegbar $karte $sonderStapel neuStapel ziel
                if {![string compare $neuStapel nixGehtMehr]} break
                set stapelKlasse [lindex $stapelBeschreibung 1]
                vorschlagSchieb 1 $sonderStapel $stapelKlasse $oben \
                                  $neuStapel $ziel $fuelle($neuStapel) 0
                if {[string match arbeit? $neuStapel]} break
                incr neuStapel
              }
            }
          }
          for {set aktStapel 0} {$aktStapel < $stapelAnz($patArt)} {incr aktStapel} {
            set oben  $teilY($aktStapel)
            if {$oben < $fuelle($aktStapel) && $oben >= $verdeckt($aktStapel)} {
              set neuStapel 0
              while {42} {
                wohinBewegbar $stapel($aktStapel,$oben) $aktStapel neuStapel ziel
                if {![string compare $neuStapel nixGehtMehr]} break

                set aktTeilStapelAnz [expr {$fuelle($aktStapel)-$oben}]
                set aktMaxTeilStapelAnz $maxTeilStapelAnz
                if {$maxTeilStapelAnz < 13} {
                  if {![string compare $ziel leerStapel]} {
                    set aktMaxTeilStapelAnz [expr {int ($aktMaxTeilStapelAnz/2)}]
                  }

                #  for {set aktY [expr $fuelle($aktStapel)-1]} {$aktY>=$oben} {incr aktY -1} {
                #    if {$fuelle($aktStapel)-$aktY > $aktMaxTeilStapelAnz} break

                #    for {set zwStapel 0} {$zwStapel < $stapelAnz($patArt)} {incr zwStapel} {
                #      if {$fuelle($zwStapel) > 0 && [string compare $zwStapel $aktStapel]
                #         && [passt $stapel($aktStapel,$oben) \
                #                   $stapel($zwStapel,[expr $fuelle($zwStapel)-1]) \
                #                   $zwStapel]} {
                #       incr aktMaxTeilStapelAnz $aktMaxTeilStapelAnz
                #      }
                #    }
                #  }
                }

                if {$aktTeilStapelAnz <= $aktMaxTeilStapelAnz} {
                  if {$oben == 0} {
                    set quelle leerStapel
                  } elseif {$oben == $verdeckt($aktStapel)
                            || ![passt $stapel($aktStapel,$oben)            \
                                       $stapel($aktStapel,[expr {$oben-1}]) \
                                       $aktStapel]} {
                    set quelle stapel
                  } else {
                    set quelle fixStapel
                  }
                  vorschlagSchieb [expr {$fuelle($aktStapel)-$oben}]   \
                                  $aktStapel $quelle $oben                     \
                                  $neuStapel $ziel $fuelle($neuStapel) \
                                  [stapelBonus $aktStapel $oben]
                }
                if {[string match arbeit? $neuStapel]} break
                incr neuStapel
              }
            }
          } 
        }
    }
    proc initVorschlaege {{neuZugNr -1}} {
      global vorschlaege prioritaet vorschlagsReihe zugNr

      if {[info exists vorschlaege] && $neuZugNr >= 0
          && $vorschlaege(zugNr) > $zugNr} return

      catch {unset vorschlaege}
      set vorschlaege(zugNr)  $neuZugNr
      set vorschlaege(anzahl) 0
      set vorschlaege(index)  0

      if {$neuZugNr >= 0} {catch {unset vorschlagsReihe}}

      if {![info exists prioritaet]} {
        set prioritaet(lohntNicht)         -1200

        set prioritaet(strafe,stock)         100
        set prioritaet(strafe,strafe)         95
        set prioritaet(strafe,talon)          85
        set prioritaet(strafe,stapel)         75
        set prioritaet(strafe,fixStapel)      73
        set prioritaet(strafe,leerStapel)     45
        set prioritaet(stapel,stock)          80
        set prioritaet(stapel,strafe)         75
        set prioritaet(stapel,talon)          65
        set prioritaet(stapel,stapel)         55
        set prioritaet(stapel,fixStapel)      30
        set prioritaet(stapel,leerStapel)     10
        set prioritaet(fixStapel,stapel)      52
        set prioritaet(fixStapel,fixStapel)   $prioritaet(lohntNicht)
        set prioritaet(fixStapel,leerStapel)   7
        set prioritaet(leerStapel,stapel)     55
        set prioritaet(leerStapel,fixStapel)  30
        set prioritaet(leerStapel,leerStapel) $prioritaet(lohntNicht)
        set prioritaet(talon,strafe)          90
        set prioritaet(talon,talon)           80
        set prioritaet(talon,stock)           75
        set prioritaet(talon,stapel)          10
        set prioritaet(talon,fixStapel)        8
        set prioritaet(talon,leerStapel)       5
        set prioritaet(arbeit,strafe)         76
        set prioritaet(arbeit,talon)          66
        set prioritaet(arbeit,stock)          81
        set prioritaet(arbeit,stapel)         11
        set prioritaet(arbeit,fixStapel)       9
        set prioritaet(arbeit,leerStapel)      6

        set prioritaet(verdeckteWirdFrei)     12
        set prioritaet(stapelWirdFrei)        25
        set prioritaet(grundBonus)            20
        set prioritaet(karteAufStock)         11
        set prioritaet(teilStapelBewegbar)     7
        set prioritaet(stapelGranularitaet)    5
        set prioritaet(andereFarbePasst)     -20
        set prioritaet(stapelNurHoch)        -13
      }
    }
    proc vorschlagDebug {key} {
      global vorschlaegeAnzeigen demoAktiv harterTest

      switch $key {
        x - q {set vorschlaegeAnzeigen 1}
        s     {set vorschlaegeAnzeigen 2}
        t     {set harterTest 1}
      }
      set demoAktiv 1
    }
    proc wertDiff {wertA wertB} {
      global aktuellerGrenzWert

      if {$wertA < $aktuellerGrenzWert && $wertB > $aktuellerGrenzWert} {
        incr wertA 13
      }
      return [expr $wertA-$wertB]
    }
    proc vorschlagAufStock {quelle quellKlasse vonY karte ziel bonus} {
      global prioritaet

      setzVorschlag $prioritaet($quellKlasse,stock)+$bonus \
          "pfeil $quelle $vonY $ziel 0" "karteAufStock $karte $quelle 1 1 0"
    }
    proc vorschlagSchieb {kartenAnz quelle quellKlasse quellY \
                                    ziel zielKlasse zielY bonus} {
      global prioritaet teilStapelReihe patArt leerStapelFeld

      if {![string compare $teilStapelReihe($patArt) garNicht]
          && $kartenAnz > 1} {
        set pfeilQuellY [expr {$quellY+$kartenAnz-1}]
        set pfeilZielY  0
        set pfeilZiel   [lindex $leerStapelFeld 0]
      } else {
        set pfeilQuellY $quellY
        set pfeilZielY  [expr {$zielY>0 ? $zielY-1 : 0}]
        set pfeilZiel   $ziel
      }
      setzVorschlag $prioritaet($quellKlasse,$zielKlasse)+$bonus \
            "pfeil $quelle $pfeilQuellY $pfeilZiel $pfeilZielY" \
            "schiebStapel $quelle $quellY $ziel $zielY [list $leerStapelFeld]"
    }
    proc wohinBewegbar {orgKarte orgStapel startStapel zielKlasse} {
      upvar $startStapel aktStapel $zielKlasse ziel
      global stapel fuelle verdeckt stapelAnz patArt \
          leerStapelNurHoch aktuellErste exist

      set moeglicherLeerStapel nixGehtMehr

      while {$aktStapel < $stapelAnz($patArt)} {
        set aktY [expr $fuelle($aktStapel)-1]

        if {$aktY < 0 && ($leerStapelNurHoch($patArt) == 0 \
            || [aufsteigend $orgKarte $aktuellErste 1])} {
          set ziel leerStapel
          set moeglicherLeerStapel $aktStapel
        }

        if {$aktY >= $verdeckt($aktStapel)} {
          if {[passt $orgKarte $stapel($aktStapel,$aktY) teil]} {
            set ziel stapel
            return
          }
          if {[passt $orgKarte $stapel($aktStapel,$aktY) $aktStapel]} {
            set ziel fixStapel
            return
          }
        }
        incr aktStapel
      }
      set aktStapel $moeglicherLeerStapel

      if {![string compare $aktStapel nixGehtMehr] &&
          ![string match arbeit? $orgStapel]} {
        for {set aktArb 0} {$aktArb < $exist($patArt,arbeit)} {incr aktArb} {
          if {[passtKarteAufStapel $startStapel arbeit$aktArb $orgKarte]} {
            set aktStapel arbeit$aktArb
            set ziel leerStapel
            return
          }
        }
      }
    }
    proc schiebStapel {vonStapel vonY neuStapel nachY leerStapelFeld} {
      global stapel fuelle zugNr teilStapelReihe patArt vorschlagsReihe auchBild

      set altFuelle $fuelle($vonStapel)
      set neuUnten  $fuelle($neuStapel)
      set kartenAnz [expr {$altFuelle-$vonY}]
      set art       [expr {$kartenAnz > 1 ? "minusStapel" : "minusEins"}]

      if {![string compare $teilStapelReihe($patArt) garNicht]
          && $kartenAnz>1 && $auchBild} {
        set zugZaehler $zugNr
        hanoi $vonStapel $neuStapel $leerStapelFeld $vonY $nachY \
                                    $kartenAnz zugZaehler

        eval $vorschlagsReihe($zugNr,schieb)
        return
      }

      switch -glob $vonStapel hand* {
        handNachTalon $vonStapel $neuStapel 1
        return
      }

      if {$auchBild} {
        schiebTeilstapel $vonStapel $vonY $neuStapel
      }
      incr fuelle($vonStapel) -$kartenAnz
      for {set aktY $vonY} {$aktY < $altFuelle} {incr aktY} {
        set aktKarte $stapel($vonStapel,$aktY)
        switch -glob $neuStapel {
          arbeit? {
            steckInStock $neuStapel $aktKarte 1 0
          }
          default {
            steckInStapel $neuStapel $aktKarte plusEinsGueltig
          }
        }
      }
      pruefFreieKarte $vonStapel $art

      incr zugNr
      if {$kartenAnz > 1} {
        protokolliereZug Stapel $vonStapel $neuUnten $neuStapel
      } elseif {$kartenAnz == 1} {
        protokolliereZug Karte $aktKarte $vonStapel $neuStapel 1
      }
    }
    proc stapelBonus {aktStapel untenY} {
      global stapel verdeckt prioritaet aktuellerGrenzWert aktuellErste\
          leerStapelNurHoch patArt exist stapelAnz teilY fuelle

      set aufSockel [expr {$untenY > $verdeckt($aktStapel)}]
      
      set wertUnten [kartenWert $stapel($aktStapel,$untenY)]
      if {$aufSockel} {
        set wertOben [kartenWert $stapel($aktStapel,[expr {$untenY-1}])]
        set bonus [wertDiff $wertUnten $wertOben]
      } else {
        switch $untenY {
          0 {
            if {$leerStapelNurHoch($patArt)} {
              set anzHoch 0
              set anzLeer 0
              for {set such 0} {$such < $stapelAnz($patArt)} {incr such} {
                if {$such == $aktStapel || $fuelle($such) == 0} {
                  incr anzLeer
                } else {
                  set hochY $teilY($such)
                  if {$hochY > 0 &&
                      [aufsteigend $stapel($such,$hochY) $aktuellErste 1]} {
                    incr anzHoch
                  }
                }
              }
              foreach sonder {talon strafe} {
                if {$exist($patArt,$sonder) > 0} {
                  set hochY [expr {$fuelle($sonder)-1}]
                  if {$hochY >= 0 && 
                      [aufsteigend $stapel($sonder,$hochY) $aktuellErste 1]} {
                    incr anzHoch
                  }
                }
              }
              if {$anzHoch < $anzLeer} {return $prioritaet(lohntNicht)}
            }
            set bonus $prioritaet(stapelWirdFrei)
          }
          1 {
            set bonus ($prioritaet(verdeckteWirdFrei)+$wertUnten)
          }
          default {
            set bonus $wertUnten
          }
        }
      }
      if {$aufSockel &&
          [kannAufStock $stapel($aktStapel,[expr {$untenY-1}])] != ""} {
        append bonus +$prioritaet(karteAufStock)
      }
      if {$aufSockel} {
        set aktUnten [expr {$untenY-1}]
        while {$aktUnten > $verdeckt($aktStapel) && \
               [passt $stapel($aktStapel,$aktUnten) \
                      $stapel($aktStapel,[expr {$aktUnten-1}]) teil]} {
          incr aktUnten -1
        }
        set neuStapel 0
        wohinBewegbar $stapel($aktStapel,$aktUnten) $aktStapel neuStapel ziel
        if {[string compare $neuStapel nixGehtMehr]
            && ![string compare $ziel stapel]} {
          # Der naechste Teilstapel kann auf einen anderen verschoben werden!
          append bonus +$prioritaet(teilStapelBewegbar)
        }
      }
      if {$aufSockel && $verdeckt($aktStapel) == 0} {
        set nurHoch 1
        for {set aktY 0} {$aktY <= $untenY && $nurHoch} {incr aktY} {
          set nurHoch [expr {[wertDiff [kartenWert $stapel($aktStapel,$aktY)] \
                                        $aktuellerGrenzWert] >= 10}]
        }
        if {$nurHoch} {
          append bonus $prioritaet(stapelNurHoch)
        }
      }
      if {$aufSockel} {
        set teilAnzahl 0
        for {set letztY $untenY} {$letztY > $verdeckt($aktStapel)} \
            {incr letztY -1} {
          set aktY [expr {$letztY-1}]
          if {![passt $stapel($aktStapel,$letztY) $stapel($aktStapel,$aktY) teil]} {
            incr teilAnzahl
          }
        }
        # Verdeckte Karten zaehlen als je ein Teilstapel.
        incr teilAnzahl $verdeckt($aktStapel)
        # Da es sowieso nur auf die Unterscheidung "1,2,3,viele" ankommt,
        # wird das jetzt hier auch so gesetzt...
        if {$teilAnzahl > 3} {set teilAnzahl 3}

        append bonus -($teilAnzahl*$prioritaet(stapelGranularitaet))
      }  
      return $bonus
    }
    proc stockBonus {karte} {
      global aktuellerGrenzWert

      return (12-[wertDiff [kartenWert $karte] $aktuellerGrenzWert])
    }
    proc hanoi {start ziel leerStapelFeld vonY nachY kartenAnz zugZaehler} {
      upvar $zugZaehler zug

      if {$kartenAnz == 1} {
        setzVorschlagsReihe $start $vonY 1 $ziel [expr {$nachY-1}] zug
      } elseif {$kartenAnz > 1} {
        set zielLeerI [lsearch $leerStapelFeld $ziel]
        if {$zielLeerI >= 0} {
          set aktLeerStapelFeld [lreplace $leerStapelFeld $zielLeerI $zielLeerI]
        } else {
          set aktLeerStapelFeld $leerStapelFeld
        }
        set grundAnz [expr {1<<([llength $aktLeerStapelFeld]-1)}]
        if {$grundAnz >= $kartenAnz} {set grundAnz [expr {$kartenAnz - 1}]}
        set tempAnz [expr {$kartenAnz-$grundAnz}]
        set tempY   [expr {$vonY+$grundAnz}]

        set leerIndex [expr {$tempAnz == 1 ? 0 : [llength $aktLeerStapelFeld]-1}]
        set erstLeer  [lindex   $aktLeerStapelFeld $leerIndex]
        set restLeer  [lreplace $aktLeerStapelFeld $leerIndex $leerIndex]

        hanoi $start $erstLeer $restLeer $tempY 0 $tempAnz zug
        hanoi $start $ziel $restLeer $vonY $nachY $grundAnz zug
        if {$vonY == 0} {lappend restLeer $start}
        hanoi $erstLeer $ziel $restLeer 0 [expr {$nachY+$grundAnz}] $tempAnz zug
      }
    }
    proc setzVorschlagsReihe {start aktY anz ziel zielY zugZaehler} {
      global vorschlagsReihe
      upvar $zugZaehler zug

      set vorschlagsReihe($zug,priori)     666
      set vorschlagsReihe($zug,prioriExpr) hanoi
      set vorschlagsReihe($zug,pfeil)      "pfeil $start $aktY $ziel $zielY"
      set vorschlagsReihe($zug,schieb)     "schiebStapel $start $aktY $ziel $zielY {}"
      incr zug
    }
    proc setzVorschlag {prioritaetExpr pfeilScript schiebScript} {
      global vorschlaege
      
      set prioritaet [expr $prioritaetExpr]
      if {$prioritaet < -1000} return

      set aktI $vorschlaege(anzahl)
      while {$aktI > 0 && $vorschlaege([expr $aktI-1],priori) < $prioritaet} {
        set letztI $aktI
        incr aktI -1
        foreach elem {priori prioriExpr pfeil schieb} {
          set vorschlaege($letztI,$elem) $vorschlaege($aktI,$elem)
        }
      }
      set  vorschlaege($aktI,priori)     $prioritaet
      set  vorschlaege($aktI,prioriExpr) $prioritaetExpr
      set  vorschlaege($aktI,pfeil)      $pfeilScript
      set  vorschlaege($aktI,schieb)     $schiebScript
      incr vorschlaege(anzahl)
    }
    proc zurueckGeben {hand} {
        global zugNr stapelAnz patArt stapel fuelle verdeckt \
               flaeche auchBild mitMischComic pos

        set letztStapel [expr $stapelAnz($patArt)-1]
        for {set aktStapel $letztStapel} {$aktStapel >= 0} {incr aktStapel -1} {
          set oben [expr $fuelle($aktStapel)-1]

          # Nach dem anfaenglichen Verteilen kann evtl. keine Karte vom
          # aktuellen Stapel genommen werden.
          if {$oben < 0} {continue}

          set karte $stapel($aktStapel,$oben)
          set stapel($hand,$fuelle($hand)) $karte

          # Nun, wo die Karte verteilt wurde, auf die Anzahl der Hand dazuzaehlen...
          # Das sollte besser schon vorm Comic passieren, da dann die Anzeige
          # der Kartenanzahl plausibler ist.
          if {!$fuelle($hand) && $auchBild} {
            stelleKarteDar $flaeche anders ruecken $hand $pos(x,$hand) $pos(y,$hand)
          }
          incr fuelle($hand)
          # ... und beim aktuellen Stapel abziehen.
          incr fuelle($aktStapel) -1

          # Den verbliebenen Stapel durchchecken und schauen, ob er expandiert
          # werden kann.
          pruefFreieKarte $aktStapel "minusEins"

          if {$auchBild} {
            # Wegschmeissen der betreffenden Spielkarte.
            if {$mitMischComic} {
              verteilComic $aktStapel $fuelle($aktStapel) $hand 0 1
            }
            $flaeche delete $karte
            update idletask
          }
        }
        incr zugNr -1
      }
    proc zurueckHandNachTalon {hand talon anzahl umsonst} {
      global zugNr runden stapel fuelle flaeche auchBild pos

      if {$anzahl == 0} {
        if {$auchBild} {
          update idletask
        }
        while {$fuelle($hand) > 0} {
          karteVonHandNach $hand $talon
        }
        if {$auchBild} {
          verteilComic $hand 0 $talon 0
        }
        incr zugNr  -1
        incr runden -1
        return
      }

      for {set aktI 0} {$aktI < $anzahl} {incr aktI} {
        incr fuelle($talon) -1
        set karte $stapel($talon,$fuelle($talon))
        if {$auchBild} {
          $flaeche delete $karte
        }
        set stapel($hand,$fuelle($hand)) $karte

        if {!$fuelle($hand) && $auchBild} {
          stelleKarteDar $flaeche anders ruecken $hand \
                         $pos(x,$hand) $pos(y,$hand)
        }
        incr fuelle($hand)
      }
      if {$auchBild} {
        verteilComic $talon 0 $hand 0
      }

      if {!$umsonst} {
        incr zugNr -1
      }
    }
    proc zurueckDrehUm {karte aktStapel} {
      global verdeckt teilStapelUnterst auchBild

      incr verdeckt($aktStapel)
      incr teilStapelUnterst($aktStapel)

      if {$auchBild} {
        maleRuecken $karte $aktStapel
        umdrehbar $karte $aktStapel
      }
    }
    proc zurueckKarte {karte neuStapel altStapel incrZugNr} {
      global stapel fuelle zugNr verdeckt auchBild

      incr fuelle($altStapel) -1
      if {[string match ? $altStapel]} {
        # Die Karte kommt von einem Stapel, also die Stapelfuelle vermindern.
        # und ueberpruefen, ob der Stapel expandiert werden kann.
        pruefFreieKarte $altStapel minusEins
      }

      if {$auchBild} {
        verteilComic $altStapel $fuelle($altStapel) \
                     $neuStapel $fuelle($neuStapel) 1
      }
      switch -glob $neuStapel {
        ?,? - arbeit? {
          steckInStock  $neuStapel $karte 1 0
        }
        default {
          # Das Folgende wird am Ende von 'zurueckDrehUm' motiviert...
          if {$auchBild && [string match ? $neuStapel] && $fuelle($neuStapel) > 0
                        && $fuelle($neuStapel) == $verdeckt($neuStapel)} {
            nichtVerschiebbar $stapel($neuStapel,[expr {$fuelle($neuStapel)-1}])
          }
          steckInStapel $neuStapel $karte plusEins
        }
      }
      if {$incrZugNr} {incr zugNr -1}
    }
    proc zurueckStapel {neuStapel altY altStapel} {
      global stapel fuelle zugNr flaeche verdeckt auchBild

      # Das Folgende wird am Ende von 'zurueckDrehUm' motiviert. Im Gegensatz zu
      # 'zurueckKarte' wissen wir hier, dass die Karten auf einen Stapel kommen.
      if {$auchBild && $fuelle($neuStapel) > 0
          && $fuelle($neuStapel) == $verdeckt($neuStapel)} {
        nichtVerschiebbar $stapel($neuStapel,[expr {$fuelle($neuStapel)-1}])
      }

      set lastI $fuelle($altStapel)
      set neuX [stapelX $neuStapel]
      if {$auchBild} {
        schiebTeilstapel $altStapel $altY $neuStapel
      }
      for {set aktI $altY} {$aktI < $lastI} {incr aktI} {
        set bewKarte $stapel($altStapel,$aktI)
        set stapel($neuStapel,$fuelle($neuStapel)) $bewKarte
        if {$auchBild} {
          $flaeche coords $bewKarte $neuX [stapelY $neuStapel $fuelle($neuStapel)]
          $flaeche raise  $bewKarte
        }
        incr fuelle($altStapel) -1
        incr fuelle($neuStapel)
      }
      pruefFreieKarte $altStapel minusStapel
      pruefFreieKarte $neuStapel plusStapel

      incr zugNr -1
    }

    proc zugZurueck {} {
      global zugNr ziehtGeradeZurueck

      if {[info exists ziehtGeradeZurueck]} return
      set ziehtGeradeZurueck 1

      if {$zugNr > 0} {zurueckZumZug [expr $zugNr-1]}

      unset ziehtGeradeZurueck
    }

    proc zumAnfang {} {
      global zugNr ziehtGeradeZurueck

      if {[info exists ziehtGeradeZurueck]} return
      set ziehtGeradeZurueck 1

      if {$zugNr > 0} {zurueckZumZug 0}

      unset ziehtGeradeZurueck
    }

    proc merken {} {
      global gemerkt marke zugNr

      set marke $zugNr
      set gemerkt 1
      # Die folgende Zuweisung ist ein indirekter Aufruf von chkZugNr.
      set zugNr $zugNr
    }

    proc zurMarke {} {
      global gemerkt marke zugNr ziehtGeradeZurueck

      if {[info exists ziehtGeradeZurueck]} return
      set ziehtGeradeZurueck 1

      if {$marke > 0} {zurueckZumZug $marke}
      if {$marke == $zugNr} {
        # Die Benutzerin hat die Funktion nicht bereits vorher abgebrochen.
        set gemerkt 0
        set marke   0
      }

      unset ziehtGeradeZurueck
    }
    proc zurueckZumZug {zielZugNr} {
      global zugNr inversZug demoAktiv auchBild harterTest

      # Kann denn ueberhaupt gerade ein Zug rueckgaengig gemacht werden?
      if {![info exists inversZug] || [llength $inversZug] == 0} {return}

      # Ab zwei zurueckzunehmenden Zuegen darf die Benutzerin jederzeit abbrechen.
      set langfristig [expr {$zugNr-$zielZugNr > 1 && $auchBild}]
      if {$langfristig && ![info exists harterTest]} {
        aktiviereBindings "set demoAktiv 0"
        menueHilfe zurueck
      }
      if {$langfristig} {
        set demoAktiv 1
      }
      if {$auchBild} {initVorschlaege}
        
      while {$zugNr > $zielZugNr && (!$langfristig || $demoAktiv)} {
        # Das letzte Element aus inversZug ausfuehren und wegschmeissen.
        set letztIndex [expr {[llength $inversZug]-1}]
        eval  [lindex $inversZug $letztIndex]
        set inversZug [lreplace $inversZug $letztIndex $letztIndex]
        if {$langfristig} {
          kurzePause
        }
      }

      if {$langfristig && ![info exists harterTest]} {
        aktiviereBindings
        menueHilfe aus
      }
    }
    proc protokolliereZug {args} {
      global inversZug streitArt patArt du text

      lappend inversZug zurueck$args
      if {$streitArt($patArt) && [string compare $du $text(computer)]} {
        eval informiereStreitGegner $args
      }
    }
    proc neuHighScore {patArt args} {
        global streitArt patiencePath text ich jetzt

        set neuZeile [concat $ich $args [list $jetzt]]
        if {$streitArt($patArt)} {
          set karten  [lindex $args 0]
        } else {
          set zuege   [lindex $args 0]
          set vorschl [lindex $args 1]
        }

        set highScore [gibHighScore $patArt]

        set zuSichern 0
        set aktInd     0
        set spielerAnz 0
        set besser     0
        foreach aktZeile $highScore {
          set aktSpieler [lindex $aktZeile 0]

          if {!$besser} {
            if {$streitArt($patArt)} {
              set besser [expr {$karten > [lindex $aktZeile 1]}]
            } else {
              set besser [expr {[zugVergl $neuZeile $aktZeile] < 0}]
            }
            if {$besser} {
              set highScore [linsert $highScore $aktInd $neuZeile]
              incr aktInd
              set  neuInd $aktInd
              set  zuSichern [expr {$spielerAnz < 4}]
              incr spielerAnz
            }
          }

          if {![string compare $ich $aktSpieler]} {
            incr spielerAnz
            if {$spielerAnz > 4} {
              set highScore [lreplace $highScore $aktInd $aktInd]
            }
          }

          incr aktInd
        }

        if {!$besser} {
          lappend highScore $neuZeile
          incr spielerAnz
          incr aktInd
          set  neuInd $aktInd
          set  zuSichern [expr {$aktInd <= 25 && $spielerAnz <= 4}]
        } elseif {$aktInd > 25} {
          set highScore [lreplace $highScore 25 [expr {$aktInd-1}]]
        }

        if {$zuSichern} {
          set meldung [schreibNeuenHighScore $patArt $highScore]
        } else {
          set meldung $text(reichtNicht)
        }

        return [list $neuInd $highScore $meldung]
      }
      proc schreibNeuenHighScore {patArt neuScore} {
        global patiencePath text

        if {[catch {source $patiencePath/patience.bst} aus] ||
            [catch {open   $patiencePath/patience.bst "w"} aus]} {
          return $aus
        } else {
          set highScore($patArt) $neuScore
          foreach patName [array names highScore] {
            puts $aus "set highScore($patName) [list $highScore($patName)]"
          }
          close $aus
          return $text(bstEingetragen)
        }
      }
    proc gibHighScore {patArt} {
      global patiencePath

      if {[catch {source $patiencePath/patience.bst}] ||
          ![info exists highScore($patArt)]} {
        return ""
      }
      return $highScore($patArt)
    }
    proc zugVergl {a b} {
      # Wurden unterschiedlich viele Vorschlaege angefordert?
      set vorschlagDiff [expr {[lindex $a 2] - [lindex $b 2]}]
      if {$vorschlagDiff} {return $vorschlagDiff}

      # Dann ist es eben die Anzahl der Zuege...
      set zugA [lindex $a 1]
      set zugB [lindex $b 1]
      return [expr {$zugA - $zugB}]
    }
    proc interneRegelNamen {} {
        global stapelAnz

        set regelList {}
        foreach el [array names stapelAnz] {
          if {![regexp ^(back|:) $el]} {
            lappend regelList $el
          }
        }
        return $regelList
      }
    proc fullPath {args} {
        if {[catch {eval file join $args} result]} {
          return [join $args /]
        }
        return $result
      }
      proc fileSelect {w ueberschrift datei dir match} {
        global fileSelect_dateiName fileSelect_dirName tk_version \
               fileSelect_match fileSelect_okay text

        catch {destroy $w}
        toplevel $w -class Dialog
        wm title $w $ueberschrift
        wm iconname $w $ueberschrift

        set fileSelect_dirName   $dir
        set fileSelect_dateiName $datei
        set fileSelect_match     $match
        set fileSelect_okay      1

        label $w.textOben   -text $ueberschrift
        pack  $w.textOben   -fill x

        frame $w.kopf -relief raised -bd 1

        frame $w.kopf.fest
        label $w.kopf.fest.dir   -text $text(fs,ordner)
        label $w.kopf.fest.datei -text $text(fs,datei)
        label $w.kopf.fest.match -text $text(fs,maske)
        pack $w.kopf.fest.dir $w.kopf.fest.datei $w.kopf.fest.match -anchor w

        frame $w.kopf.feld
        entry $w.kopf.feld.dir   -textvariable fileSelect_dirName   -width 40
        entry $w.kopf.feld.datei -textvariable fileSelect_dateiName -width 40
        entry $w.kopf.feld.match -textvariable fileSelect_match     -width 40
        pack $w.kopf.feld.dir $w.kopf.feld.datei $w.kopf.feld.match -anchor w

        pack $w.kopf.fest $w.kopf.feld -side left
        pack $w.kopf

        frame $w.box -relief raised -bd 1
        scrollbar $w.scroll -command "$w.list yview"
        pack $w.scroll -in $w.box -side right -fill y
        listbox $w.list -yscroll "$w.scroll set" -relief raised -setgrid yes
        if {$tk_version >= 4.0} {
          $w.list configure -width 30 -height 20 -selectmode single    
        } else {
          $w.list configure -geometry 30x20
          tk_listboxSingleSelect $w.list
        }
        pack $w.list -in $w.box -side left  -fill both -expand yes
        pack $w.box 
        bind $w.list <ButtonRelease-1> "+uebernimm $w.list"
        bind $w.list <Double-1>        "destroy $w"
        fillDirList $w.list $dir

        frame $w.bot -relief raised -bd 1
        pack  $w.bot -side bottom -fill both

        button $w.okay -text $text(okay) -command "destroy $w"
        frame $w.default -relief sunken -bd 1
        raise $w.okay $w.default
        pack $w.default -in $w.bot -side right -expand 1 -padx 3m -pady 2m
        pack $w.okay -in $w.default -padx 2m -pady 2m -ipadx 2m -ipady 1m

        foreach i "$w $w.kopf.feld.dir $w.kopf.feld.datei $w.kopf.feld.match" {
          bind $i <Return> "$w.okay flash; $w.okay invoke; break"
        }

        button $w.abbruch -text $text(abbruch) \
          -command "set fileSelect_okay 0; destroy $w"
        pack $w.abbruch -in $w.bot -side left -expand 1 \
                          -padx 3m -pady 3m -ipadx 2m -ipady 1m

        set oldFocus [focus]
        grab $w
        focus $w
        tkwait window $w
        focus $oldFocus

        if {$fileSelect_okay} {
          switch $fileSelect_dirName {
            "/"     {return /$fileSelect_dateiName}
            default {return $fileSelect_dirName/$fileSelect_dateiName}
          }
        } else {
          return ""
        }
      }
    proc fillDirList {list dir} {
      global fileSelect_match

      cd $dir
      set dirList [glob -nocomplain .* *]

      # Evtl. vorhandener alter Schrott wird geloescht.
      $list delete 0 end

      # In der ersten Runde alle Directories aufsammeln,
      foreach i $dirList {
        if {[file isdirectory $i]} {
          $list insert end $i
        }
      }

      # dann alle sonstigen Dateien, die zu dem gegebenen Muster passen.
      foreach i $dirList {
        if {[string match $fileSelect_match $i] && ![file isdirectory $i]} {
          $list insert end $i
        }
      }
    }
    proc uebernimm {fs} {
      global fileSelect_dateiName fileSelect_dirName

      if [catch {set datei [selection get]}] {return}

      if [file isdirectory $fileSelect_dirName/$datei] {
        if {![string compare $datei ..]
            && [regsub {/[^/]*$} $fileSelect_dirName "" neuDir]} {
          set fileSelect_dirName [expr {$neuDir == "" ? "/" : $neuDir}]
        } elseif {[string compare $datei .]} {
          set fileSelect_dirName [expr {$fileSelect_dirName == "/" ? "/$datei" :
                                    "$fileSelect_dirName/$datei"}]
        }
        fillDirList $fs $fileSelect_dirName
      } else {
        set fileSelect_dateiName $datei
      }
    }
    proc erstelltVon {} {
        global patienceVersion text

        set ausgabe    "$text(patience) / $text(version) $patienceVersion\n\n"
        append ausgabe "$text(patienceFuerX11)\n$text(erstelltVonCK)\n\n"
        append ausgabe "krischan@cs.tu-berlin.de"

        tk_dialog .t1 $text(erstellt) $ausgabe [kartenBild kk] 0 $text(okay)
      }
      proc beenden {} {
        global text

        set ausgabe    "$text(hoffeSpielWarGut)\n\n$text(wirklichBeenden)"
        tk_dialog .t1 $text(wirklichBeenden) $ausgabe "" \
                      1 $text(abbruch) $text(spielende)
      }
      proc neuesSpiel {} {
        global text

        tk_dialog .t1 $text(neuesSpiel) $text(wirklichNeu) \
                  "" 1 $text(abbruch) $text(neuesSpiel)
      }
      proc glueckwunsch {zugNr vorschlagZaehler} {
        global text

        switch $vorschlagZaehler {
          0       { set vorschlaege "" }
          1       { set vorschlaege $text(undEsGab)$text(einVorschlag) }
          default { set vorschlaege $text(undEsGab)$vorschlagZaehler$text(vorschlaege) }
        }

        set ausgabe    "$text(gratuliere)\n\n$text(patienceGeloest)\n"
        append ausgabe "$text(dazuBenoetigt)$zugNr$text(zuege)\n$vorschlaege"
        return [tk_dialog .t1 $text(glueckwunsch) $ausgabe [kartenBild dp] \
                           2 $text(spielende) $text(highScore) $text(neuesSpiel)]
      }
      proc streitGlueckwunsch {restKarten} {
        global text

        set ausgabe    "$text(gratuliere)\n\n$text(sieHabenGewonnen)"
        append ausgabe "$text(undGegnerHat)$restKarten"
        append ausgabe [expr {$restKarten>1?$text(restKarten1):$text(restKarte1)}]
        return [tk_dialog .t1 $text(glueckwunsch) $ausgabe [kartenBild dp] \
                           2 $text(spielende) $text(highScore) $text(neuesSpiel)]
      }
      proc streitBeileid {restKarten} {
        global text

        set ausgabe    "$text(schade)\n\n$text(sieHabenVerloren)"
        append ausgabe "$text(undSieHatten)$restKarten"
        append ausgabe [expr {$restKarten>1?$text(restKarten2):$text(restKarte2)}]
        tk_dialog .t1 $text(beileid) $ausgabe [kartenBild dp] 0 $text(naechstesMal)
      }
      proc tadel {zugNr mogelZaehler} {
        global text

        set ausgabe    "$text(spielAufgegangen)\n\n$text(dazuBenoetigt)"
        append ausgabe "$zugNr$text(zuege)\n$text(aberSieHaben)$mogelZaehler"
        append ausgabe "$text(malGeschummelt)\n$text(vielleichtBesser)"
        return [tk_dialog .t1 $text(glueckwunsch) $ausgabe \
                      [kartenBild bk] 1 $text(spielende) $text(probieren)]
      }
      proc nichtLesbar {dateiName} {
        global text

        tk_dialog .t1 $text(nichtLesbar) \
                      $text(dieDatei)$dateiName$text(istNichtLesbar) \
                      question 0 $text(dannNicht)
      }
      proc nichtSchreibbar {dateiName} {
        global text

        tk_dialog .t1 $text(nichtSchreibbar) \
                      $text(dieDatei)$dateiName$text(istNichtSchreibbar) \
                      question 0 $text(dannNicht)
      }
      proc nurGegenComputer {} {
        global text

        tk_dialog .t1 $text(keineVerteilung) $text(streitNurGegenComputer) \
                      [kartenBild ac] 0 $text(schade)
      }
      proc keinDaemon {fehler} {
        global text

        tk_dialog .t1 $text(keinDaemon) "'$fehler'\n$text(daemonFehler)" \
                      [kartenBild ap] 0 $text(schade)
      }
      proc keineKarten {} {
        global text

        tk_dialog .t1 $text(beenden) $text(keineKartenFonts) \
                      [kartenBild 7p] 0 $text(beenden)
      }
      proc deutschDefault {} {
        tk_dialog .t1 Achtung "Setzen der deutschen Defaulttexte..." \
                      [kartenBild 7p] 0 "Ok"
      }
    proc kartenBild {karte} {
      global bitPath aktFont kartenFont

      if {![info exists bitPath]} {return ""}

      if {![info exists kartenFont($aktFont,photo)]} {
        set path $bitPath/$karte
      } elseif {![info exists kartenFont(normal,photo)]} {
        set path $bitPath/../normal/$karte
      } else {
        return ""
      }

      if {![file exists $path]} {return ""}

      return @$path
    }
    proc startEinzelKarte {karte x y stapel stapY} {
        global lastX lastY flaeche schatten endBewegungErlaubt

        global spezialKommando
        if {[string length $spezialKommando]} {
          if {[string index $spezialKommando 0] == "+"} {
            uplevel #0 [string range $spezialKommando 1 end]
          } else {
            uplevel #0 $spezialKommando
            return
          }
        }

        $flaeche raise $karte
        spielXY lastX lastY $x $y

        if {$schatten} {
          stelleKarteDar $flaeche neu sch schatten \
              [stapelX $stapel] [stapelY $stapel $stapY]
          $flaeche lower schatten $karte
          $flaeche move $karte -$schatten -$schatten
          $flaeche itemconfigure schatten -tags "schatten $karte"
        }

        set diffX [expr {$lastX - [stapelX $stapel]}]
        set diffY [expr {$lastY - [stapelY $stapel $stapY]}]

        $flaeche bind $karte <B1-Motion> "schiebKarte $karte %x %y"
        $flaeche bind $karte <Any-ButtonRelease-1> \
                "endEinzelKarte $karte $stapel %x %y $diffX $diffY"
        set endBewegungErlaubt 1
      }
      proc drehUm {karte stapel mitZurueck} {
        global verdeckt teilStapelUnterst fuelle

        maleKarte $karte anders $stapel [expr {$fuelle($stapel)-1}]
        incr verdeckt($stapel) -1
        incr teilStapelUnterst($stapel) -1

        checkTeilStapel $stapel plusEins
        if {$mitZurueck} {
          protokolliereZug DrehUm $karte $stapel
        }
      }
      proc pruefFreieKarte {aktStapel art} {
        global fuelle stapel verdeckt streitArt patArt auchBild

        if {!$auchBild} {
          if {[string match ? $aktStapel]} {
            checkTeilStapel $aktStapel $art
          }
          return
        }

        switch -glob $aktStapel {
          ? {
            if {$fuelle($aktStapel) > 0} {
              set obenY   [expr $fuelle($aktStapel)-1]
              set oberste $stapel($aktStapel,$obenY)
              if {$fuelle($aktStapel) <= $verdeckt($aktStapel)} {
                umdrehbar $oberste $aktStapel
              } else {
                verschiebbar $oberste $aktStapel $obenY
                checkTeilStapel $aktStapel $art
                pruefStapelFuelle $aktStapel
              }
            }
          }
          talon* {
            if {$streitArt($patArt)} {
              aktiviereGeben [expr {$aktStapel=="talon2"?"hand2":"hand"}] ein
            }
            autoStock $aktStapel
          }
          strafe* {
            autoStock $aktStapel
          }
        }
      }
      proc steckInStapel {neuStapel neuKarte art} {
        global stapel fuelle auchBild

        set stapel($neuStapel,$fuelle($neuStapel)) $neuKarte
        if {$auchBild} {
          maleKarte $neuKarte alt $neuStapel $fuelle($neuStapel)
        }
        incr fuelle($neuStapel)

        if {[string match ? $neuStapel]} {
          # Passt die neue Karte auf den Stapel gut rauf?
          checkTeilStapel $neuStapel $art

          if {$auchBild} {
            # Ueberpruefen, ob der Stapel komprimiert werden muss.
            pruefStapelFuelle $neuStapel
          }
        }
      }
      proc steckInStock {neuStock neuKarte neu manuell} {
        global stapel fuelle flaeche auchBild aufStockFolge zugNr streitArt patArt

        if {$neu} {
          # Die Karte wird gerade neu auf den Stock gelegt.
          set stapel($neuStock,$fuelle($neuStock)) $neuKarte
          incr fuelle($neuStock)
        }

        if {$auchBild} {
          maleKarte $neuKarte alt $neuStock
          switch -glob $neuStock {
            ?,c     {$flaeche lower $neuKarte sh[string index $neuStock 0] }
            ?,h     {$flaeche lower $neuKarte sp[string index $neuStock 0] }
            ?,p     {$flaeche lower $neuKarte sk[string index $neuStock 0] }
            arbeit? {
              global exist patArt

              set nummer [string index $neuStock 6]
              if {$nummer < $exist($patArt,arbeit)-1} {
                $flaeche lower $neuKarte w[expr {$nummer+1}]
              }
              return
            }
          }
        }

        if {$manuell && !$streitArt($patArt)} {
          if {![info exists aufStockFolge]
              || $zugNr != $aufStockFolge(start)+$aufStockFolge(zaehler)} {
            set aufStockFolge(start)   $zugNr
            set aufStockFolge(zaehler) 1
          } elseif {[incr aufStockFolge(zaehler)] >= 12} {
            zeigHinweis ablegen
          }
        }
      }
      proc startTeilStapel {anwStapel untenY x y} {
        global stapel fuelle teilStapelUnterst lastX lastY flaeche schatten \
               endBewegungErlaubt

        global spezialKommando
        if {[string length $spezialKommando]} {
          if {[string index $spezialKommando 0] == "+"} {
            uplevel #0 [string range $spezialKommando 1 end]
          } else {
            uplevel #0 $spezialKommando
            return
          }
        }

        # Bildschirmbezogene Koordinaten auf Spielflaechen-bezogene umrechnen.
        spielXY lastX lastY $x $y

        set maxY  $fuelle($anwStapel)
        set stapX [stapelX $anwStapel]
        for {set aktY $untenY} {$aktY < $maxY} {incr aktY} {
          set karte $stapel($anwStapel,$aktY)
          if {$schatten} {
            stelleKarteDar $flaeche neu sch schatten \
              $stapX [stapelY $anwStapel $aktY]
            $flaeche lower schatten $karte
            $flaeche move $karte -$schatten -$schatten
            $flaeche itemconfigure schatten -tags "schatten teilStapel"
          }

          $flaeche raise $karte
          $flaeche itemconfigure $karte -tags "$karte teilStapel"

          nichtBewegbar $karte
        }

        # Das Bewegen und Loslassen der Maus mit den entspr. Funktionen belegen.
        set diffX [expr {$lastX - $stapX}]
        set diffY [expr {$lastY - [stapelY $anwStapel $untenY]}]
        $flaeche bind teilStapel <B1-Motion> "schiebKarte teilStapel %x %y"
        $flaeche bind teilStapel <Any-ButtonRelease-1> \
                "endTeilStapel $anwStapel $untenY %x %y $diffX $diffY"
        set endBewegungErlaubt 1
      }
      proc ablegen {} {
        global stapelAnz stockReihe patArt stapel fuelle exist teilStapelUnterst \
               hinweisGezeigt

        set gefunden 1
        set kompletteFamilie [string match *,kompl $stockReihe($patArt)]

        set hinweisGezeigt(ablegen) 1 ;# Nicht unbedingt war, aber es hilft...
        catch {global flaeche; $flaeche delete hinweis}

        while {$gefunden} {
          set gefunden 0
          if {!$kompletteFamilie} {
            set stapelListe {}
            if {$exist($patArt,talon)}  {
              lappend stapelListe talon
            }
            if {$exist($patArt,strafe)} {
              lappend stapelListe strafe
            }
            if {$exist($patArt,arbeit)} {
              for {set aktArb 0} {$aktArb < $exist($patArt,arbeit)} {incr aktArb} {
                lappend stapelListe arbeit$aktArb
              }
            }
            foreach sonderStapel $stapelListe {
              if {$fuelle($sonderStapel) > 0} {
                switch -- [karteAufStock \
                              $stapel($sonderStapel,[expr $fuelle($sonderStapel)-1]) \
                              $sonderStapel 0 1 0] {
                   1 {set gefunden 1}
                  -1 {return}
                }
              }
            }
          }
          for {set aktStapel 0} {$aktStapel<$stapelAnz($patArt)} {incr aktStapel} {
            set oberst [expr {$fuelle($aktStapel)-($kompletteFamilie?13:1)}]
            if {$oberst < $teilStapelUnterst($aktStapel)} continue

            switch -- [karteAufStock $stapel($aktStapel,$oberst) $aktStapel 0 1 0] {
               1 {set gefunden 1}
              -1 {return}
            }
          }
        }
      }
      proc tuWasGutes {event x y} {
        global tutGeradeWasGutes

        global spezialKommando
        if {[string length $spezialKommando]} {
          if {[string index $spezialKommando 0] == "+"} {
            uplevel #0 [string range $spezialKommando 1 end]
          } else {
            uplevel #0 $spezialKommando
            return
          }
        }

        if {[info exists tutGeradeWasGutes]} return
        set tutGeradeWasGutes 1

        machsBesteDraus $event $x $y

        unset tutGeradeWasGutes
      }
      proc machsBesteDraus {event x y} {
        global verdeckt stapel fuelle teilStapelUnterst flaeche endBewegungErlaubt \
               streitArt stockReihe stapelAnz talonAbraeumbar patArt

        set anwStapel [findeStapel $x $y]
        if {![string length $anwStapel]
            || (![string compare $anwStapel hand] &&
                ![string compare $event <Double-1>])} return
        catch {global flaeche; $flaeche delete hinweis}

        set oberstI   [expr $fuelle($anwStapel)-1]

        for {set aktI $oberstI} {$aktI < $fuelle($anwStapel)-1} {incr aktI} {
          $flaeche bind $stapel($anwStapel,$aktI) <Any-ButtonRelease-1> ""
        }

        set gefunden  0
        switch -glob $anwStapel {
          ? {
            if {$oberstI < $verdeckt($anwStapel)} return
            if {[string match *,kompl $stockReihe($patArt)]} {
              set oberstI [expr $fuelle($anwStapel)-13]
            }
            if {$oberstI < $teilStapelUnterst($anwStapel)} {
              set oberstI -1
            }
            set vonY $teilStapelUnterst($anwStapel)
          }
          talon* {
            if {$oberstI < $verdeckt($anwStapel) ||
                $streitArt($patArt) && $anwStapel == "talon"} return
            set vonY $oberstI
          }
          strafe* {
            if {$oberstI < 0 ||
                $streitArt($patArt) && $anwStapel == "strafe"} return
            set vonY $oberstI
          }
          hand* {
            if {[istGebenAktiv]} {geben $anwStapel}
            return
          }
          arbeit? {
            if {$oberstI < 0} return
            set vonY $oberstI
          }
          default return
        }
        catch {$flaeche delete schatten}
        if {$oberstI >= $vonY} {
          if {[karteAufStock $stapel($anwStapel,$oberstI) $anwStapel 0 1 1]} {
            set gefunden 1
            set neuStapel x,x
          }
        }

        if {!$gefunden} {
          set miesesZiel ""
          if {[string match ? $anwStapel]} {
            set neuStapel [expr {$anwStapel+1}]
            if {$neuStapel<$stapelAnz($patArt)} {
              while {42} {
                wohinBewegbar $stapel($anwStapel,$vonY) $anwStapel neuStapel ziel
                if {![string compare $neuStapel nixGehtMehr]} break
                if {![string compare $ziel stapel]} {
                  schiebStapel $anwStapel $vonY $neuStapel $fuelle($neuStapel) {}
                  set gefunden 1
                  break
                }
                if {![string length $miesesZiel]
                    || ($miesesZiel == "leerStapel" && $ziel != "leerStapel")} {
                  set miesesZiel   $ziel
                  set mieserStapel $neuStapel
                }
                if {[string match arbeit? $neuStapel]} break
                incr neuStapel
              }
            }
          }
          if {!$gefunden} {
            set neuStapel 0
            while {42} {
              wohinBewegbar $stapel($anwStapel,$vonY) $anwStapel neuStapel ziel
              if {![string compare $neuStapel nixGehtMehr]} break
              if {![string compare $ziel stapel]} {
                schiebStapel $anwStapel $vonY $neuStapel $fuelle($neuStapel) {}
                set gefunden 1
                break
              }
              if {![string length $miesesZiel]
                  || ($miesesZiel == "leerStapel" && $ziel != "leerStapel")} {
                set miesesZiel   $ziel
                set mieserStapel $neuStapel
              }
              if {[string match arbeit? $neuStapel]} break
              incr neuStapel
            }
          }
        }

        if {!$gefunden && [string length $miesesZiel]} {
          schiebStapel $anwStapel $vonY $mieserStapel $fuelle($mieserStapel) {}
          set gefunden 1
        }

        if {!$gefunden} {
          global tk_version
          if {$tk_version >= 4.0} {
            bell
          } else {
            puts -nonewline "\a"
          }
        } else {
          catch {unset endBewegungErlaubt}
          if {[string match ?,? $neuStapel] || $streitArt($patArt) || !$fuelle(hand)} {
            if {[checkSpielEnde 0]} return
          }
          switch -glob $anwStapel {
            ? {
              if {$fuelle($anwStapel) == $verdeckt($anwStapel)
                  && $fuelle($anwStapel) > 0} {
                drehUm $stapel($anwStapel,[expr {$fuelle($anwStapel)-1}]) $anwStapel 1
              }
            }
            talon* {
              if {!$talonAbraeumbar($patArt)
                  && $fuelle($anwStapel) == $verdeckt($anwStapel) } {
                handNachTalon [expr {$anwStapel == "talon" ? "hand" : "hand2"}] $anwStapel 1
              }
            }
          }
        }
      }
      proc karteAufStock {anwKarte anwStapel inDemo incrZugNr manuell} {
        global fuelle zugNr flaeche stockReihe patArt auchBild streitArt

        set neuStock [kannAufStock $anwKarte]
        if {![string length $neuStock]} {return 0}

        if {$incrZugNr} {incr zugNr}

        switch -glob $stockReihe($patArt) {
          *,kompl {
            set vonY [expr {$fuelle($anwStapel)-13}]
            schiebTeilstapel $anwStapel $vonY $neuStock
            plaziereTeilstapel $anwStapel $neuStock $vonY
            protokolliereZug Stapel $anwStapel 0 $neuStock
            set art minusStapel
          }
          default {
            incr fuelle($anwStapel) -1
            protokolliereZug Karte $anwKarte $anwStapel $neuStock $incrZugNr
            if {$auchBild} {
              verteilComic $anwStapel $fuelle($anwStapel) $neuStock 0 1
            }
            steckInStock $neuStock $anwKarte 1 $manuell
            set art minusEins
          }
        }
        if {$auchBild} {
          pruefFreieKarte $anwStapel $art; update idletask
          if {!$streitArt($patArt) && [checkSpielEnde $inDemo]} {return -1}
        }
        return 1
      }
      proc kannAufStock {altKarte} {
        global kartenSatz patArt

        set aktFarbe [string index $altKarte 1]
        for {set aktSatz 0} {$aktSatz < $kartenSatz($patArt)} {incr aktSatz} {
          set aktStock $aktSatz,$aktFarbe
          if [passt $altKarte [getStock $aktStock] $aktStock] {
            return $aktStock
          }
        }
        return ""
      }
      proc geben {hand} {
        global zugNr stapelAnz exist streitArt patArt stapel fuelle stapelGrund \
               verdeckt flaeche mitMischComic pos aktiverSpieler ich

        catch {global flaeche; $flaeche delete hinweis}

        if {![string length $hand]} {
          set neu  1
          if {$streitArt($patArt)} {
            set hand [expr {$aktiverSpieler == $ich ? "hand2" : "hand"}]
          } else {
            set hand hand
          }
        } else {
          if {$streitArt($patArt)} {
            global aktiverSpieler ich du

            if {![string compare $aktiverSpieler $ich] && [string compare $hand hand2]} {
              return 0
            }
            if {![string compare $aktiverSpieler $du]  && [string compare $hand hand]} {
              return 0
            }
          }
          set neu  0
          if {$exist($patArt,talon) > 0} {
            set talon [expr {$hand == "hand" ? "talon" : "talon2"}]
            return [handNachTalon $hand $talon 1]
          }
        }

        for {set aktStapel 0} {$aktStapel<$stapelAnz($patArt)} {incr aktStapel} {
          # Es werden nur Karten verteilt, wenn noch welche da sind. Das muss in
          # jedem Durchgang zugesichert werden, da nicht bei allen Patiencen
          # fuer jeden Stapel immer noch eine Karte in der Hand ist.
          if {$fuelle($hand) == 0} {return 0}

          set aktHoehe $fuelle($aktStapel)

          # Beim anfaenglichen Verteilen braucht evtl. keine Karte auf den
          # aktuellen Stapel gelegt werden.
          if {$neu && $aktHoehe >= $stapelGrund($patArt,$aktStapel)} {
            continue
          }

          if {$mitMischComic} {
            verteilComic $hand 0 $aktStapel $fuelle($aktStapel)
          }

          # Oberste Karte vom Stapel nehmen und in den Zielstapel stecken.
          set karte $stapel($hand,[incr fuelle($hand) -1])
          set stapel($aktStapel,$aktHoehe) $karte

          if {!$fuelle($hand)} {
            stelleKarteDar $flaeche anders h $hand $pos(x,$hand) $pos(y,$hand)
          }

          # Auf dem Bildschirm darstellen (evtl. auf dem Ruecken).
          if {$fuelle($aktStapel) < $verdeckt($aktStapel)} {
            maleRuecken $karte $aktStapel $fuelle($aktStapel)
          } else {
            maleKarte $karte neu $aktStapel $fuelle($aktStapel)
          }
          update idletask
          incr fuelle($aktStapel)

          if {$fuelle($aktStapel) > $verdeckt($aktStapel)} {
            pruefFreieKarte $aktStapel plusEins

            # Evtl. werden Karten automatisch auf den Stock geschoben.
            autoStock $aktStapel
          }

          # Hiermit wird erreicht, das die Karten immer abwechselnd von den
          # Haenden der beiden Spielparteien genommen wird.
          if {$neu && $streitArt($patArt)} {
            set hand [expr {$hand == "hand" ? "hand2" : "hand"}]
          }
        }

        if {!$neu} {
          incr zugNr
          protokolliereZug Geben $hand
        }
        return 0
      }
      proc handNachTalon {hand talon mitZurueck} {
        global zugNr stapelAnz exist patArt stapel fuelle runden pos \
               flaeche streitArt talonAbraeumbar auchBild verdeckt durchgaenge

        if {$fuelle($hand) == 0} {
          if {$fuelle($talon) == 0} {return 0}
          if {$durchgaenge($patArt)>=0 && $runden>=$durchgaenge($patArt)} {return 0}
          incr runden
          while {$fuelle($talon) > 0} {
            incr fuelle($talon) -1
            set karte $stapel($talon,$fuelle($talon))
            if {$auchBild} {$flaeche delete $karte}
            set stapel($hand,$fuelle($hand)) $karte
            incr fuelle($hand)
          }
          if {$auchBild} {
            stelleKarteDar $flaeche anders ruecken $hand $pos(x,$hand) $pos(y,$hand)
            verteilComic $talon 0 $hand 0
          }
          set verdeckt($talon) 0
          if {$mitZurueck} {
            protokolliereZug HandNachTalon $hand $talon 0 0
          }
          set umgepackt 1
        } else {
          set umgepackt 0
        }

        set anzahl $exist($patArt,talon)
        if {!$talonAbraeumbar($patArt) && $fuelle($talon) > 0 && $auchBild} {
          set verdeckt($talon) $fuelle($talon)
          nichtVerschiebbar $stapel($talon,[expr {$fuelle($talon)-1}])
        }
        if {$auchBild} {verteilComic $hand 0 $talon 0}
        if {$fuelle($hand) < $anzahl} {set anzahl $fuelle($hand)}
        for {set aktI 0} {$aktI < $anzahl} {incr aktI} {
          karteVonHandNach $hand $talon
        }
        if {$streitArt($patArt)} {
          aktiviereGeben $hand aus
        }
        if {$auchBild} {update idletask}

        if {$mitZurueck} {
          incr zugNr
          protokolliereZug HandNachTalon $hand $talon $anzahl $umgepackt
        }
        if {$anzahl > 0 && $auchBild} {
          return [autoStock $talon]
        } else {
          return 0
        }
      }
      proc karteVonHandNach {hand dest} {
        global fuelle stapel flaeche auchBild pos

        set karte $stapel($hand,[incr fuelle($hand) -1])
        set stapel($dest,$fuelle($dest)) $karte

        # Nun, wo die Karte verteilt wurde, von der Anzahl der Hand abziehen...
        # Das sollte besser schon vorm Comic passieren, da dann die Anzeige
        # der Kartenanzahl plausibler ist.
        
        if {!$fuelle($hand) && $auchBild} {
          stelleKarteDar $flaeche anders h $hand $pos(x,$hand) $pos(y,$hand)
        }
        # ... und beim Ziel (Talon, Strafstapel oder dergl.) wieder daraufaddieren.
        incr fuelle($dest)

        # Anlegen und Aufdecken der Spielkarte an der gewuenschten Stelle.
        if {$auchBild} {maleKarte $karte neu $dest}
      }
      proc schluss {rueckfrage} {
        global demoAktiv kommandoNachDemo streitArt patArt du text

        if {$rueckfrage} {
          if {$streitArt($patArt) && [string compare $du $text(computer)]} {
            if {![kontaktBeenden 0]} return
          } elseif {![beenden]} {
            return
          }
        }

        if {$demoAktiv} {
          set demoAktiv 0
          set kommandoNachDemo "exit"
        }
        exit
      }
      proc starteSpiel {mitRueckfrage {anderePat ""}} {
        global neuPat streitArt kartenSatz patArt zugNr flaeche pos \
               text demoAktiv ich du streitVorgabe aktiverSpieler   \
               kommandoNachDemo spielStartetGerade mogelZaehler vorschlagZaehler

        if {$mitRueckfrage > 0} {
          if {$streitArt($patArt) && [string compare $du $text(computer)]} {
            if {![kontaktBeenden [expr ![string compare $neuPat $patArt]]]} {
              set neuPat $patArt; return
            }
          } elseif {$zugNr > 0} {
            if {![neuesSpiel]} {set neuPat $patArt; return}
          }
        }
        if {$mitRueckfrage && $demoAktiv} {
          set demoAktiv        0
          set kommandoNachDemo "starteSpiel 0 $anderePat"
          return
        }
        if {[string length $anderePat]} {
          set neuPat $anderePat
        }

        set andereRegeln [string compare $patArt $neuPat]
        set patArt $neuPat

        if {[info exists spielStartetGerade]} {
          tkwait variable spielStartetGerade
        }
        set spielStartetGerade 1

        if {$andereRegeln} {
          highScore 0
        }

        if {!$demoAktiv} {
          # Solange wir das Bild aufbauen, darf keine Aufforderung zur Demo stoeren.
          aktiviereKritischeKommandos aus
        }

        # Alle Items der Spielflaeche wegschmeissen und leere Flaeche malen.
        $flaeche delete all
        update

        initZustand

        # Um sicherzustellen, dass alles gemaess der evtl. neuen Regeln auf
        # den Bildschirm passt.
        setBitPath <>

        maleLeereStoecke

        # Zunaechst das tatsaechliche Mischen der Karten in die Haende anstossen.
        mischen
        if {$streitArt($patArt)} {
          kontaktDialogEnde

          if {[string compare $du $text(computer)]
              && ![info exists streitVorgabe]} {
            global stapel fuelle patArt text ich du kontakt meinRechner

            foreach srcDest {{deineHand hand} {meineHand hand2}} {
              set dest [lindex $srcDest 0]
              set src  [lindex $srcDest 1]
              set $dest $fuelle($src)
              for {set aktI 0} {$aktI < $fuelle($src)} {incr aktI} {
                lappend $dest $stapel($src,$aktI)
              }
            }

            senden "
              set streitVorgabe(du)          $ich
              set streitVorgabe(deinRechner) $meinRechner
              set streitVorgabe(deinStart)   $stapel(meinStart)
              set streitVorgabe(meinStart)   $stapel(deinStart)
              set streitVorgabe(deineHand)   [list $meineHand]
              set streitVorgabe(meineHand)   [list $deineHand]
              after 0 starteSpiel -1 $patArt
            "
          }
          catch {unset streitVorgabe}
          aktiviereMenuepunkteFuerStreit
          werFaengtAn
          $flaeche bind schachUhr <1> "streitZugBestaetigung"
        } else {
          aktiviereMenuepunkteFuerStreit
        }

        # Jetzt den oder die Kartenstapel beim Mischen zeichnen.
        mischComic $pos(x,hand) $pos(y,hand) hand links
        if {$streitArt($patArt)} {
          mischComic $pos(x,hand2) $pos(y,hand2) hand2 rechts
        }

        # Das kleine Fensterchen mit der Anzahl der Karten.
        maleKartenAnz

        global kartenSatz stapelGrund stapelVerdeckt stapelAnz patArt \
               aktuellErste streitArt stapel verdeckt fuelle          \
               teilStapelUnterst exist stockBasis flaeche pos         \
               aktuellerGrenzWert talonAbraeumbar mitMischComic

        update idletask

        set maxGrund 0
        for {set aktStapel 0} {$aktStapel < $stapelAnz($patArt)} {incr aktStapel} {
          set fuelle($aktStapel) 0

          set verdeckt($aktStapel)          $stapelVerdeckt($patArt,$aktStapel)
          set teilStapelUnterst($aktStapel) $stapelVerdeckt($patArt,$aktStapel)
          if {$maxGrund < $stapelGrund($patArt,$aktStapel)} {
            set maxGrund $stapelGrund($patArt,$aktStapel)
          }
        }
        for {set aktSatz 0} {$aktSatz < $kartenSatz($patArt)} {incr aktSatz} {
          foreach aktFarbe {c h p k} {
            set fuelle($aktSatz,$aktFarbe) 0
          }
        }
        switch $stockBasis($patArt) {
          erste {
            set karte $stapel(hand,[incr fuelle(hand) -1])
            set aktStock 0,[string index $karte 1]

            # Verteilen, Anlegen und Aufdecken der Karte an der gewuenschten Stelle.
            verteilComic hand 0 $aktStock 0
            maleKarte    $karte neu $aktStock
            steckInStock $aktStock $karte 1 0
            set aktuellErste       [string index $karte 0]
            set aktuellerGrenzWert [kartenWert $karte]
          } 
          default {
            set aktuellErste $stockBasis($patArt)
            set aktuellerGrenzWert [kartenWert $stockBasis($patArt)]
          }
        }

        if {$exist($patArt,talon) > 0} {
          set fuelle(talon)    0
          set fuelle(talon2)   0
          set verdeckt(talon)  0
          set verdeckt(talon2) 0
        }

        if {$exist($patArt,strafe) > 0} {
          # Aufbau des Strafstapels.
          set fuelle(strafe)    0
          set fuelle(strafe2)   0
          set verdeckt(strafe)  0
          set verdeckt(strafe2) 0

          verteilComic hand 0 strafe 0
          for {set aktI 0} {$aktI < $exist($patArt,strafe)} {incr aktI} {
            karteVonHandNach hand strafe
          }
          if {$streitArt($patArt)} {
            verteilComic hand2 0 strafe2 0
            for {set aktI 0} {$aktI < $exist($patArt,strafe)} {incr aktI} {
              karteVonHandNach hand2 strafe2
            }
          }
          # Evtl. werden Karten automatisch auf den Stock geschoben.
          autoStock strafe
        }

        for {set aktArb 0} {$aktArb < $exist($patArt,arbeit)} {incr aktArb} {  
          set fuelle(arbeit$aktArb) 0
          set verdeckt(arbeit$aktArb) 0
        }

        for {set aktStapel 0} {$aktStapel <= $maxGrund} {incr aktStapel} {  
          geben ""
        }

        if {$exist($patArt,talon) > 0 && $talonAbraeumbar($patArt)} {
          handNachTalon hand talon 0
          if {$streitArt($patArt)} {
            handNachTalon hand2 talon2 0
          }
        }

        if {$exist($patArt,arbeit)} {
          set restAnz $fuelle(hand)
          if {$restAnz > 0} {
            set restHand hand
          }
          if {$streitArt($patArt) && $fuelle(hand2) > 0} {
            incr restAnz $fuelle(hand2)
            lappend restHand hand2
          }
          if {$restAnz <= $exist($patArt,arbeit)} {
            set aktArb [expr {int($exist($patArt,arbeit)/(2*$restAnz))}]
            foreach hand $restHand {
              while {$fuelle($hand)} {
                incr fuelle($hand) -1
                if {$mitMischComic} {
                  verteilComic $hand 0 arbeit$aktArb 0
                }
                set karte $stapel($hand,$fuelle($hand))
                maleKarte $karte neu arbeit$aktArb
                steckInStock arbeit$aktArb $karte 1 0
                incr aktArb
              }
            }
            stelleKarteDar $flaeche anders h $hand $pos(x,$hand) $pos(y,$hand)
          }
        }

        unset spielStartetGerade
        if {$streitArt($patArt)} {
          confMenueEntry .menu.karten.m $text(geben) -command "geben hand2"
          if {![string compare $aktiverSpieler $ich]} {
            schaltSchachuhrAufMensch 1
          } else {
            streitZugBestaetigung
          }
        } elseif {!$demoAktiv} {
          aktiviereBindings
        }

        if {!$demoAktiv} {
          aktiviereKritischeKommandos ein
        }
      }
    proc schiebKarte {karte x y} {
      global lastX lastY flaeche

      spielXY neuX neuY $x $y
      $flaeche move $karte [expr $neuX-$lastX] [expr $neuY-$lastY]
      set lastX $neuX
      set lastY $neuY
    }
    proc endEinzelKarte {karte anwStapel x y diffX diffY} {
      global stapel fuelle zugNr veraendern mogelZaehler stockReihe \
             streitArt talonAbraeumbar patArt flaeche endBewegungErlaubt

      if {![info exists endBewegungErlaubt]} return
      catch {$flaeche delete schatten}

      if {[string match ?,? $anwStapel] && !$talonAbraeumbar($patArt)} {
        set treffer 0
      } else {
        set neuStapel [findeStapel $x $y]
        switch -glob $neuStapel {
          ? - talon - strafe - arbeit* {
            set treffer [passtKarteAufStapel $anwStapel $neuStapel $karte]
            if {$veraendern && !$treffer && [string compare $anwStapel $neuStapel]} {
              set treffer 1
              incr mogelZaehler
            }
          }
          ?,? {
            set treffer [expr {![string match *,kompl $stockReihe($patArt)]
                               && [passt $karte [getStock $neuStapel] $neuStapel]
                               && $anwStapel != $neuStapel}]
          }
          default {
            set treffer 0
          }
        }
        if {$treffer} {
          switch -glob $neuStapel {
            ? - talon* - strafe* {
              steckInStapel $neuStapel $karte plusEinsGueltig
            }
            ?,? - arbeit? {
              steckInStock $neuStapel $karte 1 1
            }
          }
        }
      }
      if {$treffer} {
        incr fuelle($anwStapel) -1
        incr zugNr
        protokolliereZug Karte $karte $anwStapel $neuStapel 1

        pruefFreieKarte $anwStapel minusEins
        if {[string match ?,? $neuStapel] || $streitArt($patArt) || !$fuelle(hand)} {
          if {[checkSpielEnde 0]} return
        }
        if {$streitArt($patArt)} {
          deaktiviereGegnerStapel
        }
      } else {
        set nachY [expr {$fuelle($anwStapel)-1}]
        verteilComic $anwStapel $nachY $anwStapel $nachY 1 \
                     [expr {$x-$diffX}] [expr {$y-$diffY}]
        switch -glob $anwStapel {
          ?,? - arbeit? {steckInStock $anwStapel $karte 0 0}
          default       {maleKarte $karte alt $anwStapel $nachY}
        }
      }
      nichtBewegbar $karte
    }
    proc passtKarteAufStapel {anwStapel neuStapel karte} {
      global patArt stapel fuelle verdeckt leerStapelNurHoch aktuellErste

      if {![string compare $anwStapel $neuStapel]} {return 0}

      switch -glob $neuStapel {
        talon*  {set umgedreht 0}
        arbeit? {return [expr {$fuelle($neuStapel) == 0}]}
        default {set umgedreht $verdeckt($neuStapel)}
      }
      if {$fuelle($neuStapel) <= $umgedreht} {
        if {$fuelle($neuStapel) == 0} {
          if {![string match ? $neuStapel]} {return 0}
          if {$leerStapelNurHoch($patArt)} {
            return [aufsteigend $karte $aktuellErste 1]
          } else {
            return 1
          }
        } else {
          return 0
        }
      }

      set neuKarte $stapel($neuStapel,[expr $fuelle($neuStapel)-1])
      return [passt $karte $neuKarte $neuStapel]
    }
    proc checkTeilStapel {neuStapel aenderung} {
      global stapel verdeckt fuelle teilStapelUnterst patArt \
             stapelReihe teilStapelReihe auchBild

      set ungueltigSetzen 1
      set neuStapelX      [stapelX $neuStapel]
      set oben            [expr $fuelle($neuStapel)-1]
      switch $aenderung {
        plusEins - plusStapel {
          set unten $teilStapelUnterst($neuStapel)
        }
        minusEins - minusStapel {
          set unten $verdeckt($neuStapel)
        }
        plusStapelGueltig {
          set unten $teilStapelUnterst($neuStapel)
          if {![string compare $stapelReihe($patArt) $teilStapelReihe($patArt)]} {
            set ungueltigSetzen 0
          }
        }
        plusEinsGueltig {
          set unten [expr $oben-1]
          if {$unten < 0} {return}
          if {![string compare $stapelReihe($patArt) $teilStapelReihe($patArt)]} {
            set ungueltigSetzen 0
          } elseif {$unten > $teilStapelUnterst($neuStapel)} {
            set unten $teilStapelUnterst($neuStapel)
          }
        }
        default {return}
      }

      set passtNoch 1
      set verglI    $oben
      while {$verglI > $unten} {
        set verglKarte $stapel($neuStapel,$verglI)
        incr verglI -1
        set unterKarte $stapel($neuStapel,$verglI)
        if {$passtNoch && ![passt $verglKarte $unterKarte teil]} {
          if {!$ungueltigSetzen} {return}
          set passtNoch 0
          set teilStapelUnterst($neuStapel) [expr $verglI+1]
        }
        if {$auchBild} {
          if {$passtNoch} {
            teilStapelBar $unterKarte $neuStapel $verglI
          } else {
            nichtVerschiebbar $unterKarte
          }
        }
      }
      if {$passtNoch && $ungueltigSetzen} {
        set teilStapelUnterst($neuStapel) $unten
      }
    }
    proc endTeilStapel {anwStapel untenY x y diffX diffY} {
      global stapel fuelle zugNr flaeche patArt streitArt endBewegungErlaubt

      if {![info exists endBewegungErlaubt]} return

      catch {$flaeche delete schatten}

      set karte $stapel($anwStapel,$untenY)
      set neuStapel [findeStapel $x $y]
      switch -glob $neuStapel {
        ?   {set treffer [passtKarteAufStapel $anwStapel $neuStapel $karte]}
        ?,? {set treffer [expr {$fuelle($anwStapel)==$untenY+13 &&
                                [passt $karte [getStock $neuStapel] $neuStapel]}]}
        default {set treffer 0}
      }
      if {$treffer} {
        # Der Stapel wurde an einen gueltigen Ort verschoben.
        set neuY $fuelle($neuStapel)
        plaziereTeilstapel $anwStapel $neuStapel $untenY
        # Wurde eine bisher verdeckte Karte nun freigelegt oder verdeckt, und
        # muss der Stapel expandiert oder komprimiert werden?
        pruefFreieKarte $anwStapel minusStapel
        pruefFreieKarte $neuStapel plusStapelGueltig

        incr zugNr
        protokolliereZug Stapel $anwStapel $neuY $neuStapel
        if {[string match ?,? $neuStapel] || $streitArt($patArt) || !$fuelle(hand)} {
          if {[checkSpielEnde 0]} return
        }
      } else {
        set lastI $fuelle($anwStapel)
        verteilComic $anwStapel $untenY $anwStapel $untenY [expr {$lastI-$untenY}] \
                [expr {$x-$diffX}] [expr {$y-$diffY}]
        for {set aktI $untenY} {$aktI < $lastI} {incr aktI} {
          $flaeche coords $stapel($anwStapel,$aktI) [stapelX $anwStapel] \
                                                    [stapelY $anwStapel $aktI]
          }
        set obenY [expr $fuelle($anwStapel)-1]
        verschiebbar $stapel($anwStapel,$obenY) $anwStapel $obenY
      }
      $flaeche dtag teilStapel teilStapel
    }
    proc schiebTeilstapel {vonStapel vonY neuStapel} {
      global stapel fuelle mitBewegComic

      if {$mitBewegComic} {
        set kartenAnz [expr {$fuelle($vonStapel) - $vonY}]
        verteilComic $vonStapel [expr {$fuelle($vonStapel)-$kartenAnz}] \
                     $neuStapel $fuelle($neuStapel) $kartenAnz
      }
    }
    proc plaziereTeilstapel {anwStapel neuStapel anwTeilStapelNr} {
      global stapel fuelle flaeche auchBild

      set lastI $fuelle($anwStapel)
      set neuX [stapelX $neuStapel]
      for {set aktI $anwTeilStapelNr} {$aktI < $lastI} {incr aktI} {
        set bewKarte $stapel($anwStapel,$aktI)
        set stapel($neuStapel,$fuelle($neuStapel)) $bewKarte
        incr fuelle($anwStapel) -1
        switch -glob $neuStapel {
          ?,? {
            steckInStock $neuStapel $bewKarte 1 1
            if {$auchBild} {
              nichtVerschiebbar $bewKarte
            }
          }
          default {
            if {$auchBild} {
              $flaeche coords $bewKarte $neuX \
                    [stapelY $neuStapel $fuelle($neuStapel)]
            }
            incr fuelle($neuStapel)
          }
        }
      }
    }
    proc getStock {stock} {
      global stapel fuelle

      set oberst [expr $fuelle($stock)-1]
      return [expr {$oberst < 0 ? "" : $stapel($stock,$oberst)}]
    }
    proc checkSpielEnde {inDemo} {
      global fuelle kartenSatz streitArt stockReihe patArt ich du text \
             zugNr mogelZaehler vorschlagZaehler
      
      if {$streitArt($patArt)} {
        set computerAnz 0
        set menschAnz   0
        foreach sonderStapel {hand talon strafe} {
          incr computerAnz $fuelle($sonderStapel)
          incr menschAnz   $fuelle(${sonderStapel}2)
        }
        if {!$computerAnz} {
          streitBeileid $menschAnz
        } elseif {!$menschAnz} {
          if {[string compare $du $text(computer)]} {
            informiereStreitGegner Beileid $computerAnz
          }
          set auswahl [streitGlueckwunsch $computerAnz]
          if {!$auswahl} {schluss 0}

          # Neuaufbau der Bestenliste, falls sie gerade angezeigt wird.
          pruefSpielerNameUndDatum
          highScore [expr {$auswahl == 1}] \
                    [neuHighScore $patArt $computerAnz $du]
        } else {
          return 0
        }
      } else {
        if {$fuelle(hand) > 0} {return 0}
        if {[string match *,kompl $stockReihe($patArt)]} {
          global teilStapelUnterst stapelAnz

          set komplettAnz 0
          for {set aktStapel 0} {$aktStapel < $stapelAnz($patArt)} {incr aktStapel} {
            if {$fuelle($aktStapel) == 13 && !$teilStapelUnterst($aktStapel)} {
              incr komplettAnz
            } elseif {($fuelle($aktStapel))} {
              return 0
            }
          }
          if {$komplettAnz && $komplettAnz != 4*$kartenSatz($patArt)} {return 0}
        } else {
          for {set aktSatz 0} {$aktSatz < $kartenSatz($patArt)} {incr aktSatz} {
            foreach aktFarbe {c h p k} {
              if {$fuelle($aktSatz,$aktFarbe) < 13} {return 0}
            }
          }
        }
        if {!$inDemo} {
          global zugNr vorschlagZaehler mogelZaehler
          if {$mogelZaehler > 0} {
            if {![tadel $zugNr $mogelZaehler]} {schluss 0}
          } else {
            set auswahl [glueckwunsch $zugNr $vorschlagZaehler]
            if {!$auswahl} {schluss 0}

            # Neuaufbau der Bestenliste, falls sie gerade angezeigt wird.
            pruefSpielerNameUndDatum
            highScore [expr {$auswahl == 1}] \
                      [neuHighScore $patArt $zugNr $vorschlagZaehler]
          }
        }
      }

      if {$inDemo} {
        starteSpielInDemo
      } else {
        starteSpiel 0
      }
      return 1
    }
    proc autoStock {aktStapel} {
      global stockBasis patArt stapel fuelle verdeckt aktuellErste demoAktiv
      
      # Werden denn ueberhaupt Karten automatisch auf den Stock geschoben?
      if {$demoAktiv || [string compare $stockBasis($patArt) erste]} {return 0}

      set gefunden 0
      while {42} {
        set aktFuelle $fuelle($aktStapel)
        if {$aktFuelle <= $verdeckt($aktStapel)} {return $gefunden}
        
        set oberste $stapel($aktStapel,[expr $aktFuelle-1])
        if {[string compare $aktuellErste [string index $oberste 0]]} {
          return $gefunden
        }
        
        if {[karteAufStock $oberste $aktStapel 0 0 0] != 1} {return $gefunden}
        
        set gefunden 1
      }
    }
    proc initPatience {} {
      global patiencePath demoAktiv auchBild \
             veraendern gemerkt marke zugNr runden fuelle streitGegner

      set demoAktiv     0
      set veraendern    0
      set gemerkt       0
      set marke         0
      set auchBild      1
      set zugNr        -1
      set runden        0
      set streitGegner computer

      # fuelle(hand) wird hier tricksigerweise mit 1 initialisiert, damit die Hand
      # nicht beim erstenmal kurzfristig leer dargestellt wird.
      set fuelle(hand) 1

      # Den Zufallszahlengenerator warmlaufen lassen.
      if {[info tclversion] < 7.5} {
        set seed [pid]
      } else {
        set seed [expr {abs([pid]+[clock clicks]%100000)}]
      }
      initr $seed

      # Die Spielregeln in der entsprechenden Directory einlesen.
      liesSpiel
    }
    # 0. Einige Kleinigkeiten vorweg.
    if {![info exists tcl_platform]} {
      set tcl_platform(platform) unix
    }
    # Nutz Die C-Library, wenn's geht...
    lappend auto_path $patiencePath
    if {[catch {load $patiencePath/Pat[info sharedlibextension]} msg]} {
      source $patiencePath/Pat.tcl
    }
    # 1. Als Vorarbeit die Spielflaeche erzeugen,
    mkFlaeche
    # 2. dann die anderen nicht ganz unwichtigen Vorarbeiten machen...
    initPatience
    mkMenue
    fuellFlaeche
    # 3. Mit Nachdruck dafuer sorgen, dass auch alle Vorarbeiten erledigt sind...
    wm deiconify .
    update
    if {![info exists bildBreite]} {tkwait variable bildBreite}
    # 4. ...und gib das erste Mal
    starteSpiel 0
